import tkinter as tk
from tkinter import ttk
import platform
import numpy as np
import warnings
from tkinter import colorchooser
import time
import string

class ParamWidgetFrame(ttk.Frame):
    '''
    基础参数控件的类型。所有的参数控件都在其上派生而来。
    '''

    def __init__(self, parent):
        super().__init__(parent)
        self.on_para_change = None
        self.__app = None  # SciApp。初始化控件的时候指定，并且在调用set_app的时候传入。

    def para_changed(self):
        if (self.on_para_change is not None) and (self.__app is not None):
            self.on_para_change(self.__app)
            print(time.time())

    def set_app(self, app):
        self.__app = app

    def is_key(self,event,type=''):
        '''
        'dir':判断方向键
        'alpha':判断是否为26个字母
        'hex':判断是否为十六进制数字或者字母
        'digit':判断是否为数字0~9
        'valid':包含数字、字母或者退格键。
        '''
        print(event.keysym)
        type = type.lower()
        if type=='':
            return True
        elif type.startswith('dir'):
            return event.keysym.lower() in ('left','right','up','down')
        elif type.startswith('alpha'):
            return event.keysym in  string.ascii_lowercase
        elif type.startswith('hex'):
            return event.keysym in string.hexdigits
        elif type.startswith(('digit')):
            return event.keysym in string.digits



class NumCtrl(ParamWidgetFrame):
    """NumCtrl: derived from tk.Entry
    用于输入数值。
    """

    def __init__(self, parent, rang: tuple, accury: int, title: str, unit: str):
        super().__init__(parent)
        self.on_check_callback = None
        self.prefix = lab_title = ttk.Label(self, text=title)
        self.prefix.pack(fill=tk.X, expand=1)

        entryFrame = ttk.Frame(self)

        self.ctrl = tk.Entry(entryFrame)
        self.ctrl.bind('<KeyRelease>', self.ontext, '+')
        self.ctrl.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        self.postfix = lab_unit = ttk.Label(entryFrame, text=unit)
        self.postfix.pack(side=tk.LEFT, fill=tk.X, expand=1)
        entryFrame.pack(expand=1, fill=tk.X)

        self.min, self.max = rang
        self.accury = accury

    def Bind(self, z, f):
        self.f = f

    def ontext(self, event):
        self.f(self)
        if self.GetValue() == None:
            self.ctrl.config(bg='#ffff00')
        else:
            self.ctrl.config(bg='#ffffff')
            self.para_changed()
        self.ctrl.update()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def SetValue(self, n):
        self.ctrl.delete(0, tk.END)
        self.ctrl.insert(0, str(round(n, self.accury) if self.accury > 0 else int(n)))

    def GetValue(self):
        sval = self.ctrl.get()
        print(sval)
        try:
            num = float(sval) if self.accury > 0 else int(sval)
        except ValueError:
            import traceback
            traceback.print_exc()
            return None
        if num < self.min or num > self.max:
            return None
        if abs(round(num, self.accury) - num) > 10 ** -(self.accury + 5):  # 这么写才比较严谨吧
            return None
        return num

    def f(self, e):
        pass

    def Refresh(self):
        pass


class TextCtrl(ParamWidgetFrame):
    def __init__(self, parent, title, unit):
        super().__init__(parent)
        self.on_check_callback = None

        self.prefix = lab_title = ttk.Label(self, text=title)
        self.prefix.pack(fill=tk.X, expand=1)

        entryFrame = ttk.Frame(self)

        self.ctrl = tk.Entry(entryFrame)
        self.ctrl.bind('<KeyRelease>', self.ontext)
        self.ctrl.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        self.postfix = lab_unit = ttk.Label(entryFrame, text=unit)
        self.postfix.pack(side=tk.LEFT, fill=tk.X, expand=1)
        entryFrame.pack(expand=1, fill=tk.X)

    def param_changed(self, event):
        pass

    # ! TODO: what is this?
    def Bind(self, z, f): self.f = f

    def ontext(self, event):
        self.para_changed()

    def SetValue(self, n: str):
        self.ctrl.delete(0, tk.END)
        self.ctrl.insert(0, n)

    def GetValue(self) -> str:
        return self.ctrl.get()


class ColorCtrl(ParamWidgetFrame):
    def __init__(self, parent, title, unit):
        super().__init__(parent)
        self.on_check_callback = None
        self.prefix = lab_title = ttk.Label(self, text=title)
        self.prefix.pack(fill=tk.X, expand=1)

        entryFrame = ttk.Frame(self)

        self.ctrl = tk.Entry(entryFrame)
        self.ctrl.bind('<KeyRelease>', self.ontext)
        self.ctrl.bind('<Double-Button-1>', self.oncolor)
        self.ctrl.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        self.postfix = lab_unit = ttk.Label(entryFrame, text=unit)
        self.postfix.pack(side=tk.LEFT, fill=tk.X, expand=1)
        entryFrame.pack(expand=1, fill=tk.X)

    def Bind(self, z, f):
        self.f = f

    def ontext(self, event):
        print('value ', self.GetValue())
        if self.GetValue() == None:
            self.ctrl.config(bg='#ffffff')
        else:
            self.ctrl.config(bg=self.ctrl.get())
            self.para_changed()
        self.ctrl.update()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def oncolor(self, event):
        rst = None
        val = self.GetValue()
        print('val', val)
        color = self.colorTup2Str(self.GetValue())
        if color is None:
            color = '#ffffff'
        color = colorchooser.askcolor(color)  # wx.ColourDialog(self)
        print(color)
        self.SetValue(self.colorStr2Tup(color[1]))
        if callable(self.on_check_callback):
            self.on_check_callback()

    ##        dialog.GetColourData().SetChooseFull(True)
    ##        if dialog.ShowModal() == wx.ID_OK:
    ##            rst = dialog.GetColourData().GetColour()
    ##            self.ctrl.SetBackgroundColour(rst)
    ##            self.ctrl.SetValue(rst.GetAsString(wx.C2S_HTML_SYNTAX))
    ##            self.f(self)
    ##        dialog.Destroy()

    def SetValue(self, color):
        print(color)
        if color is None:
            color = (255, 255, 255)
        strcolor = self.colorTup2Str(color)
        self.ctrl.config(bg=strcolor)  # SetBackgroundColour(color)
        self.ctrl.delete(0, tk.END)
        self.ctrl.insert(0, strcolor)

    def GetValue(self):
        rgb = self.ctrl.get().strip()
        print('rgnb', rgb)
        if len(rgb) != 7 or rgb[0] != '#':
            return None
        try:
            int(rgb[1:], 16)
        except:
            import traceback
            traceback.print_exc()
            return None
        return self.colorStr2Tup(rgb)

    def colorStr2Tup(self, value: str) -> tuple:  # pos或者wh的输入都是tuple
        def convert(c):
            v = ord(c)
            if (48 <= v <= 57):
                return v - 48
            else:
                return v - 87  # 返回a的值。

        value = value.lower()
        c0 = convert(value[1]);
        c1 = convert(value[2])
        c2 = convert(value[3]);
        c3 = convert(value[4])
        c4 = convert(value[5]);
        c5 = convert(value[6])
        a1 = c0 * 16 + c1;
        a2 = c2 * 16 + c3;
        a3 = c4 * 16 + c5
        return (a1, a2, a3)

    def colorTup2Str(self, value: tuple) -> str:
        strcolor = '#'
        for i in value:
            strcolor += hex(int(i))[-2:].replace('x', '0')
        return strcolor


class PathCtrl(ParamWidgetFrame):
    def __init__(self, parent, title, filt):
        super().__init__(parent)
        self.prefix = lab_title = ttk.Label(self, text=title)
        self.prefix.pack(fill=tk.X, expand=1)

        entryFrame = ttk.Frame(self)
        self.ctrl = tk.Entry(entryFrame)
        self.ctrl.bind('<KeyRelease>', self.ontext)
        self.ctrl.bind('<ButtonPress-1>', self.onselect)
        self.ctrl.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        entryFrame.pack(expand=1, fill=tk.X)

    def Bind(self, z, f): self.f = f

    def ontext(self, event):
        self.para_changed()
        print('ColorCtrl')

    def onselect(self, event):
        # [TODO]:对应的这个函数是啥意思？一定要注意一下！
        pass

    def SetValue(self, value: str):
        self.ctrl.delete(0, tk.END)
        self.ctrl.insert(0, value)

    def GetValue(self) -> str:
        return self.ctrl.get()


class Choice(ParamWidgetFrame):
    def __init__(self, parent, choices, tp, title, unit):
        super().__init__(parent)
        self.tp, self.choices = tp, choices
        self.on_check_callback = None

        self.prefix = lab_title = ttk.Label(self, text=title)
        self.prefix.pack(anchor=tk.W)

        self.sel = tk.IntVar()
        self.sel.set(0)
        for i, choice in enumerate(self.choices):
            b = ttk.Radiobutton(self, text=str(choice),
                                variable=self.sel, value=i, command=self.on_choice)
            b.pack(anchor=tk.W)

        self.postfix = lab_unit = ttk.Label(self, text=unit)
        self.postfix.pack(side=tk.LEFT, fill=tk.X, expand=1)

    def f(self, v):
        pass

    def Bind(self, z, f):
        self.f = f

    def on_choice(self, event=None):
        # attention : button command will not transfer any event as args .
        # 注意：按钮本身并不会传递event作为参数，与键鼠的event不同。
        self.f(self)
        print(self.GetValue())
        self.para_changed()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def SetValue(self, x):
        n = self.choices.index(x) if x in self.choices else 0
        self.sel.set(n)

    def GetValue(self):
        return self.choices[self.sel.get()]  # self.tp(self.choices[self.ctrl.GetSelection()])


class AnyType(ParamWidgetFrame):
    def __init__(self, parent, title, types=['Int', 'Float', 'Str']):
        wx.Panel.__init__(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.Size(-1, -1),
                          style=wx.TAB_TRAVERSAL)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.prefix = lab_title = wx.StaticText(self, wx.ID_ANY, title,
                                                wx.DefaultPosition, wx.DefaultSize)
        lab_title.Wrap(-1)
        sizer.Add(lab_title, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.txt_value = wx.TextCtrl(self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0)
        sizer.Add(self.txt_value, 1, wx.ALIGN_CENTER | wx.ALL, 5)
        com_typeChoices = types
        self.postfix = self.com_type = wx.ComboBox(self, wx.ID_ANY, 'Float', wx.DefaultPosition, wx.DefaultSize,
                                                   com_typeChoices, 0)
        sizer.Add(self.com_type, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.SetSizer(sizer)
        self.Layout()

        # Connect Events
        self.txt_value.Bind(wx.EVT_KEY_UP, self.on_text)
        self.com_type.Bind(wx.EVT_COMBOBOX, self.on_type)

    def Bind(self, z, f):
        self.f = f

    def SetValue(self, v):
        self.txt_value.SetValue(str(v))
        if isinstance(v, int):
            self.com_type.Select(0)
        if isinstance(v, float):
            self.com_type.Select(1)
        else:
            self.com_type.Select(2)

    def GetValue(self):
        tp = self.com_type.GetValue()
        sval = wx.TextCtrl.GetValue(self.txt_value)
        if tp == 'Float':
            try:
                num = float(sval)
            except ValueError:
                return None
        if tp == 'Int':
            try:
                num = int(sval)
            except ValueError:
                return None
        if tp == 'Str':
            try:
                num = str(sval)
            except ValueError:
                return None
        return num

    # Virtual event handlers, overide them in your derived class
    def on_text(self, event):
        self.f(self)
        if self.GetValue() == None:
            self.txt_value.SetBackgroundColour((255, 255, 0))
        else:
            self.txt_value.SetBackgroundColour((255, 255, 255))
        self.Refresh()

    def on_type(self, event):
        if self.GetValue() == None:
            self.txt_value.SetBackgroundColour((255, 255, 0))
        else:
            self.txt_value.SetBackgroundColour((255, 255, 255))
        self.Refresh()


class Choices(ParamWidgetFrame):
    def __init__(self, parent, choices, title):
        self.choices = list(choices)
        super().__init__(parent)

        self.prefix = lab_title = ttk.Label(self, text=title)
        self.prefix.pack(anchor=tk.W)

        self.boolVarList = [tk.BooleanVar() for i in range(len(self.choices))]
        # print(self.boolVarList)
        self.on_check_callback = None
        for i, choice in enumerate(self.choices):
            b = ttk.Checkbutton(self, text=str(choice),
                                variable=self.boolVarList[i], command=self.on_check)
            b.pack(anchor=tk.W)

    def Bind(self, z, f):
        self.f = f

    def on_check(self):
        self.para_changed()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def GetValue(self):
        l = []
        for i, bv in enumerate(self.boolVarList):
            if bv.get() == True:
                l.append(self.choices[i])
        return l

    def SetValue(self, value: list):
        print('set value', value)
        for bv in self.boolVarList: bv.set(False)
        for i, v in enumerate(value):
            n = self.choices.index(v) if v in self.choices else -1  # -1 stands for input error
            if n != -1:
                self.boolVarList[n].set(True)
            else:
                warnings.warn('Index item \'%s\' is not in choices list : %s' %
                              (v, repr(self.choices)))


class FloatSlider(ParamWidgetFrame):
    def __init__(self, parent, rang, accury, title, unit=''):
        super().__init__(parent)
        self.accury = accury
        self.on_check_callback = None

        self.lab_title = ttk.Label(self, text=title)
        self.lab_title.pack(anchor=tk.W)

        self.slider = ttk.Scale(self, from_=rang[0], to=rang[1], orient=tk.HORIZONTAL, command=self.on_scroll)
        self.slider.pack(expand=1, fill=tk.X)
        self.spinboxFrame = ttk.Frame(self)

        self.lab_min = ttk.Label(self.spinboxFrame)
        self.lab_min.pack(side=tk.LEFT)

        self.spinvar = tk.StringVar()
        self.spin = tk.Spinbox(self.spinboxFrame, from_=rang[0], to=rang[1], increment=10 ** -accury, format='%10.4f',
                               command=self.on_spin, textvariable=self.spinvar)
        self.spin.bind('<KeyRelease>', self.on_text)

        self.spin.pack(side=tk.LEFT)
        self.spinboxFrame.pack(expand=1, fill=tk.BOTH, pady=(3, 0))

        self.lab_max = ttk.Label(self.spinboxFrame)
        self.lab_max.pack(side=tk.LEFT)

        self.lab_unit = ttk.Label(self.spinboxFrame, text=unit)
        self.lab_unit.pack(side=tk.LEFT)

        self.set_para(rang, accury)

    def Bind(self, z, f):
        self.f = f

    def set_para(self, rang, accury):
        self.min = round(rang[0], accury)
        self.max = round(rang[1], accury)
        self.lab_min.config(text=str(round(rang[0], accury)))
        self.lab_max.config(text=str(round(rang[1], accury)))
        self.accury = accury

    def on_scroll(self, event):
        '''
        只有这个方法会调用回调函数。原因：当用户通过拖动滚动条进行操作的时候，输入的值一定是合理的，无需进行判断；
        当用户通过按spinbox或者是对spinbox输入文本的时候，只有文本符合规定的时候会对slider设置值，此时会调用这个方法。
        这样总能调用到这个回调函数。
        '''
        value = self.slider.get()
        self.spinvar.set(round(value, self.accury))
        self.spin.config(bg='#ffffff')
        self.para_changed()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def on_spin(self):

        self.slider.set(self.spin.get())
        if callable(self.on_check_callback):
            self.on_check_callback()

    def on_text(self, event):
        if self.is_key(event,'dir'):
            return
        if self.GetValue() == None:
            self.spin.config(bg="#ffff00")
        else:
            self.spin.config(bg="#ffffff")
            self.SetValue(self.GetValue())

    def SetValue(self, n):
        self.slider.set(n)
        self.spinvar.set(round(n, self.accury))

    def GetValue(self):
        sval = self.spinvar.get()
        try:
            val = float(sval) if self.accury > 0 else int(float(sval))
            if self.min <= val <= self.max:
                return val
            else:
                return None
        except Exception as e:
            print(e)
            #raise Exception('Invalid sval', sval)


class Label(ParamWidgetFrame):
    def __init__(self, parent, title):
        super().__init__(parent)
        lab_title = ttk.Label(self, text=title)
        lab_title.pack(anchor=tk.W)

    def Bind(self, z, f): pass

    def SetValue(self, v): pass

    def GetValue(self, v): pass


class Check(ParamWidgetFrame):
    def __init__(self, parent, title):
        super().__init__(parent)
        lab_title = ttk.Label(self, text=title)
        lab_title.pack(anchor=tk.W)
        self.on_check_callback = None
        self.variable = tk.BooleanVar()
        checkFrame = ttk.Frame(self)
        check = ttk.Checkbutton(checkFrame, command=self.on_check, variable=self.variable)
        check.pack(side=tk.LEFT)
        checkFrame.pack(expand=True, fill=tk.X)

    def GetValue(self):
        return self.variable.get()

    def SetValue(self, value):
        self.variable.set(value)

    def on_check(self):
        ##        print(self.variable.get())
        ##        print(self.GetValue())
        self.para_changed()
        if callable(self.on_check_callback):
            self.on_check_callback()
        pass

    def Bind(self, z, f): self.f = f


if __name__ == '__main__':
    view = [('lab', 'lab', 'This is a questionnaire'),
            (str, 'name', 'name', 'please'),
            (int, 'age', (0, 150), 0, 'age', 'years old'),
            (float, 'h', (0.3, 250000), 8, 'height', 'm'),
            ('slide', 'w', (1, 150), 0, 'weight', 'kg'),
            (bool, 'sport', 'do you like sport'),
            (list, 'sys', ['Windows', 'Mac', 'Linux'], str, 'favourite', 'system'),
            ('chos', 'lan', ['C/C++', 'Java', 'Python'], 'lanuage you like(multi)'),
            ('color', 'c', 'which', 'you like')]
    root = tk.Tk()

    ##    v=view[3]
    ##    print(v,v[2:])
    ##    nc=NumCtrl(root, *v[2:])
    ##    nc.SetValue(123.456)
    ##    nc.pack(fill=tk.BOTH,expand=True)

    ##    v=view[1]
    ##    tc=TextCtrl(root,*v[2:])
    ##    tc.SetValue('123.456')
    ##    tc.pack(fill=tk.BOTH,expand=True)

    ##    v=view[8]
    ##    cc=ColorCtrl(root,*v[2:])
    ##    cc.pack(fill=tk.BOTH,expand=True)

    ##    v = view[6]
    ##    print(v)
    ##    cc=Choice(root,*v[2:])
    ##    cc.pack(fill=tk.BOTH,expand=True)
    ##    cc.SetValue('Mac')

    v = view[3]
    print(v)
    cc = Check(root, *v[2:])
    cc.pack(fill=tk.BOTH, expand=True)
    cc.SetValue(False)
    ##    cc.SetValue(['Java'])
    ##    cc.SetValue(['C/C++','Python','C--'])

    root.mainloop()



from .shpbase import pick_obj, pick_point, drag, offset
from .. import ImageTool, ShapeTool
from ...object import Point, Line, Polygon, Layer, Points, Texts
from ...object.roi import *
import numpy as np
from numpy.linalg import norm

def mark(shp, types = 'all'):
    pts = []
    if not (types=='all' or shp.dtype in types): return pts
    if shp.dtype == 'point': pts.append([shp.body])
    if shp.dtype == 'line': pts.append(shp.body)
    if shp.dtype == 'polygon': pts.append(shp.body[0])
    if shp.dtype == 'layer':
        minl, obj = 1e8, None
        for i in shp.body:
            pts.extend(mark(i, types))
    return pts

class BaseEditor(ShapeTool):
    def __init__(self, dtype='all'):
        self.status, self.oldxy, self.p = '', None, None
        self.pick_m, self.pick_obj = None, None

    def mouse_down(self, shp, x, y, btn, **key):
        self.p = x, y
        if btn==2:
            self.status = 'move'
            self.oldxy = key['px'], key['py']
        if btn==1 and self.status=='pick':
            m, obj, l = pick_point(shp, x, y, 5)
            self.pick_m, self.pick_obj = m, obj
        if btn==1 and self.pick_m is None:
            m, l = pick_obj(shp, x, y, 5)
            self.pick_m, self.pick_obj = m, None
        if btn==3:
            obj, l = pick_obj(shp, x, y, 5)
            if key['alt'] and not key['ctrl']:
                if obj is None: del shp.body[:]
                else: shp.body.remove(obj)
                shp.measure_mark()
                shp.dirty = True
            if not (key['shift'] or key['alt'] or key['ctrl']):
                key['canvas'].fit()

    def mouse_up(self, shp, x, y, btn, **key):
        self.status = ''
        if btn==1:
            self.pick_m = self.pick_obj = None
            if not (key['alt'] and key['ctrl']): return
            pts = mark(shp)
            if len(pts)>0: 
                pts = Points(np.vstack(pts), color=(255,0,0))
                key['canvas'].marks['anchor'] = pts
            shp.dirty = True

    def mouse_move(self, shp, x, y, btn, **key):
        self.cursor = 'arrow'
        if self.status == 'move':
            ox, oy = self.oldxy
            up = (1,-1)[key['canvas'].up]
            key['canvas'].move(key['px']-ox, (key['py']-oy)*up)
            self.oldxy = key['px'], key['py']
        if key['alt'] and key['ctrl']:
            self.status = 'pick'
            if not 'anchor' in key['canvas'].marks: 
                pts = mark(shp)
                if len(pts)>0: 
                    pts = Points(np.vstack(pts), color=(255,0,0))
                    key['canvas'].marks['anchor'] = pts
            if 'anchor' in key['canvas'].marks:
                m, obj, l = pick_point(key['canvas'].marks['anchor'], x, y, 5)
                if not m is None: self.cursor = 'hand'
        elif 'anchor' in key['canvas'].marks: 
            self.status = ''
            del key['canvas'].marks['anchor']
            shp.dirty = True
        if not self.pick_obj is None and not self.pick_m is None:
            drag(self.pick_m, self.pick_obj, x, y)
            shp.measure_mark()
            pts = mark(self.pick_m)
            if len(pts)>0:
                pts = np.vstack(pts)
                key['canvas'].marks['anchor'] = Points(pts, color=(255,0,0))
            self.pick_m.dirty = True
            shp.dirty = True
        if self.pick_obj is None and not self.pick_m is None:
            offset(self.pick_m, x-self.p[0], y-self.p[1])
            pts = mark(self.pick_m)
            if len(pts)>0:
                pts = np.vstack(pts)
                key['canvas'].marks['anchor'] = Points(pts, color=(255,0,0))
            shp.measure_mark()
            self.p = x, y
            self.pick_m.dirty = shp.dirty = True

    def mouse_wheel(self, shp, x, y, d, **key):
        if d>0: key['canvas'].zoomout(x, y, coord='data')
        if d<0: key['canvas'].zoomin(x, y, coord='data')

class BaseMeasure(ImageTool):
    def __init__(self, base): 
        base.__init__(self)
        self.base = base

    def mouse_down(self, img, x, y, btn, **key):
        if img.mark is None: img.mark = Measure()
        self.base.mouse_down(self, img.mark, x, y, btn, **key)

    def mouse_up(self, img, x, y, btn, **key):
        self.base.mouse_up(self, img.mark, x, y, btn, **key)
        if not img.mark is None:
            if len(img.mark.body)==0: img.mark = None

    def mouse_move(self, img, x, y, btn, **key):
        self.base.mouse_move(self, img.mark, x, y, btn, **key)

    def mouse_wheel(self, img, x, y, d, **key):
        self.base.mouse_wheel(self, img.mark, x, y, d, **key)

def inbase(key, btn):
    status = key['ctrl'], key['alt'], key['shift']
    return status == (1,1,0) or btn in {2,3}

class PointEditor(BaseEditor):
    title = 'Point Tool'
    def __init__(self): 
        BaseEditor.__init__(self)

    def mouse_down(self, shp, x, y, btn, **key):
        if inbase(key, btn):
            BaseEditor.mouse_down(self, shp, x, y, btn, **key)
        if btn==1 and not key['alt'] and not key['ctrl']:
            shp.body.append(Coordinate([x,y]))
            shp.measure_mark()
            shp.dirty = True

class LineEditor(BaseEditor):
    title = 'Line Tool'
    def __init__(self, tp): 
        BaseEditor.__init__(self)
        self.cur, self.n, self.obj, self.tp = 0, 0, None, tp

    def mouse_down(self, shp, x, y, btn, **key):
        if inbase(key, btn) and self.obj is None:
            BaseEditor.mouse_down(self, shp, x, y, btn, **key)
        if key['alt'] and key['ctrl']: return
        if btn==1:
            if self.obj is None: 
                self.obj = self.tp([(x,y)])
                shp.body.append(self.obj)
            else: 
                self.obj.body = np.vstack((self.obj.body, [(x,y)]))
            anchor = Points(mark(self.obj)[0], color=(255,0,0))
            key['canvas'].marks['buffer'] = anchor
        if btn==3 and not self.obj is None :
            self.obj.body = np.vstack((self.obj.body, [(x,y)]))
            self.obj.dirty, shp.dirty, self.obj = True, True, None
            del key['canvas'].marks['buffer']
        shp.measure_mark()
        shp.dirty = True

class AreaEditor(BaseEditor):
    title = 'Polygon Tool'
    def __init__(self): 
        BaseEditor.__init__(self)
        self.cur, self.n, self.obj = 0, 0, None

    def mouse_down(self, shp, x, y, btn, **key):
        if inbase(key, btn) and self.obj is None:
            BaseEditor.mouse_down(self, shp, x, y, btn, **key)
        if key['alt'] and key['ctrl']: return
        if btn==1:
            if self.obj is None: 
                self.obj = Line([(x,y)])
                shp.body.append(self.obj)
            else: 
                self.obj.body = np.vstack((self.obj.body, [(x,y)]))
            anchor = Points(mark(self.obj)[0], color=(255,0,0))
            key['canvas'].marks['buffer'] = anchor
        if btn==3 and not self.obj is None :
            body = np.vstack((self.obj.body, [(x,y)]))
            shp.body[-1] = Area(body)
            shp.measure_mark()
            self.obj, shp.dirty = None, True
            del key['canvas'].marks['buffer']
        shp.dirty = True

class DistanceEditor(LineEditor):
    def __init__(self): LineEditor.__init__(self, Distance)

class AngleEditor(LineEditor):
    def __init__(self): LineEditor.__init__(self, Angle)

class SlopEditor(LineEditor):
    def __init__(self): LineEditor.__init__(self, Slope)

class CoordinateTool(BaseMeasure):
    title = 'Coordinate Tool'
    def __init__(self): 
        BaseMeasure.__init__(self, PointEditor)

class DistanceTool(BaseMeasure):
    title = 'Distance Tool'
    def __init__(self): 
        BaseMeasure.__init__(self, DistanceEditor)

class AngleTool(BaseMeasure):
    title = 'Angle Tool'
    def __init__(self): 
        BaseMeasure.__init__(self, AngleEditor)

class SlopeTool(BaseMeasure):
    title = 'Slop Tool'
    def __init__(self): 
        BaseMeasure.__init__(self, SlopEditor)

class AreaTool(BaseMeasure):
    title = 'Area Tool'
    def __init__(self):
        BaseMeasure.__init__(self, AreaEditor)


from .shape import *
import numpy as np
from numpy.linalg import norm
import shapely.geometry as geom

class ROI(Layer):
	default = {'color':(255,255,0), 'fcolor':(255,255,255), 
    'fill':False, 'lw':1, 'tcolor':(255,0,0), 'size':8}

	def __init__(self, body=None, **key):
		if isinstance(body, Layer):  body = body.body
		if not body is None and not isinstance(body, list):
			body = [body]
		Layer.__init__(self, body, **key)
		self.fill = False
		self.msk = None

	@property
	def roitype(self):
		roitype = ''
		for i in self.body:
			if roitype == '': roitype = i.dtype
			if roitype != i.dtype: return 'multi'
		return roitype

	def to_mask(self, msk, mode):
		from ..util import draw_shp
		if isinstance(msk, tuple):
			msk = np.zeros(msk, dtype=np.int8)
		else: msk[:] = 0
		msk.dtype = np.int8
		if mode=='in':
			draw_shp(self.to_geom(), msk, 1, 0)
			np.clip(msk, 0, 1, out=msk)
		if mode=='out':
			draw_shp(self.to_geom(), msk, 1, 0)
			np.clip(msk, 0, 1, out=msk)
			msk -= 1
		if isinstance(mode, int):
			draw_shp(self.to_geom(), msk, 1, mode)
			np.clip(msk, 0, 1, out=msk)
		msk.dtype, self.msk = np.bool, mode
		return msk

class Measure(Layer): 
	default = {'color':(255,255,0), 'fcolor':(255,255,255), 
	'fill':False, 'lw':1, 'tcolor':(255,0,0), 'size':8}

	def measure(self, shp):
		txt = []
		if shp.dtype == 'layer':
			for i in shp.body:
				txt.extend(self.measure(i))
		if not hasattr(shp, 'mtype'): return txt
		if shp.mtype=='coordinate':
			txt.append(tuple(shp.body)+('%.2f,%.2f'%tuple(shp.body),))
		if shp.mtype=='distance':
			for s,e in zip(shp.body[:-1], shp.body[1:]):
				p = ((s[0]+e[0])/2, (s[1]+e[1])/2)
				l = ((s[0]-e[0])**2+(s[1]-e[1])**2)**0.5
				txt.append(p+('%.2f'%l,))
		if shp.mtype=='angle':
			pts = np.array(shp.body)
			v1 = pts[:-2]-pts[1:-1]
			v2 = pts[2:]-pts[1:-1]
			a = np.sum(v1*v2, axis=1)*1.0
			a/=norm(v1,axis=1)*norm(v2,axis=1)
			ang = np.arccos(a)/np.pi*180
			for v, p in zip(ang, shp.body[1:-1]):
				txt.append(tuple(p[:2])+('%.2f'%v,))
		if shp.mtype=='slope':
			pts = np.array(shp.body)
			mid = (pts[:-1]+pts[1:])/2

			dxy = (pts[:-1]-pts[1:])
			dxy[:,1][dxy[:,1]==0] = 1
			l = norm(dxy, axis=1)*-np.sign(dxy[:,1])
			ang = np.arccos(dxy[:,0]/l)/np.pi*180
			for v, p in zip(ang, mid):
				txt.append(tuple(p[:2])+('%.2f'%v,))
		if shp.mtype=='area':
			geom = shp.to_geom()
			o = geom.centroid
			txt.append((o.x, o.y)+('%.2f'%geom.area,))
		return txt

	def measure_mark(self):
		txt = self.measure(self)
		if len(txt)>0:
			rms = [i for i in self.body if isinstance(i, Texts)]
			for i in rms: self.body.remove(i)
			self.body.append(Texts(txt))

class Coordinate(Point): mtype = 'coordinate'

class Distance(Line): mtype = 'distance'

class Area(Polygon): mtype = 'area'

class Angle(Line): mtype = 'angle'

class Slope(Line): mtype = 'slope'

if __name__ == '__main__':
	pts = Points([(10,10),(15,20)])
	line = Line([(10,10),(15,20),(20,20)])
	lines = Lines([[(10,10),(15,20),(20,20)]])
	polygon = Polygon([[(1,1),(1,20),(20,20),(20,1)],[(5,5),(5,10),(10,10),(10,5)]])

	im = draw_shp(pts.to_geom(), (30,30), lw=0)
	plt.imshow(im)
	plt.show()



import tkinter as tk
from tkinter import ttk
import cv2
import PIL.Image, PIL.ImageTk
import numpy as np
from sciapp.action import Tool, DefaultTool
from scitk.canvas.boxutil import cross, multiply, merge, lay, mat, like
from time import time
from scitk.canvas.imutil import mix_img

class Canvas(tk.Canvas):
    scales = [0.03125, 0.0625, 0.125, 0.25, 0.5, 0.75, 1, 1.5, 2, 3, 4, 5, 8, 10, 15, 20, 30, 50]
    def __init__(self,parent,autofit=False, ingrade=False, up=False):
        super().__init__(parent)
#        self.image=None
##        self.imgtmp=None
##        self.imgpos=(0,0)

        self.winbox = [0,0,300,300] # 窗口
        self.conbox = [0,0,1,1]
        self.oribox = [0,0,1,1]
        
        self.outbak = None
        self.outimg = None
        self.outrgb = None
        self.outbmp = None
        self.outint = None
        self.buffer = None

        self.first = True

        self.images = []
        self.marks = {}
        self.tool = None
        
        self.scaidx = 6
        self.autofit = autofit
        self.ingrade = ingrade
        self.up = up
        self.scrbox = (parent.winfo_screenwidth(),parent.winfo_screenheight())# 屏幕大小

        self.fps=0
        self.t1=self.t0=time()
        self.bindEvents()
        
### 以下为测试代码之后要删去
        self.tool=DefaultTool()
        
        self.imageShownList=[]# tkinter 需要对画布上的对象保持引用才能显示。否则是不会显示的。

    def draw_rgb(self, image, x, y):
        self.imgtmp = PIL.ImageTk.PhotoImage(image = PIL.Image.fromarray(image))
        self.create_image(x ,y ,image=self.imgtmp, anchor=tk.NW)
        
    def get_obj_tol(self):return self,Tool.default
        
        
    def bindEvents(self):
         self.bind('<Configure>', self.on_size)
         self.bind('<MouseWheel>',self.on_mouse_wheel)
         self.bind('<ButtonPress>',self.on_mouse_down)
         self.bind('<ButtonRelease>',self.on_mouse_up)
         self.bind('<B1-Motion>',self.on_mouse_drag)
    
    def _get_btn_and_others(self,event):# 获取点击鼠标时候按键的输入。
        px, py = event.x, event.y
        self._keydic={17:[False,False,True], 20:[False,True,False], 
                      4:[False,True,True],16:[False]*3,
                    272:[False]*3,276:[False,True,False],273:[False,False,True]
                    ,277:[False,True,True]}
        if event.state in self._keydic.keys(): sta = self._keydic[event.state]
        else: sta=[False]*3
        others = {'alt':sta[0], 'ctrl':sta[1], 'shift':sta[2], 'px':px, 'py':py, 'canvas':self}
        return event.num,others
        
    def on_mouse_move(self,event):
        ename=event.type._name_.lower()
        key,btn=self._get_btn_and_key(event)
        pass
        
    def on_mouse_up(self,event):
        x,y=self.to_data_coor(event.x, event.y)
        btn,others=self._get_btn_and_others(event)
        self.tool.mouse_up(self,  x,  y, btn, **others)    
        self.update()

    def on_mouse_drag(self,event):
        x,y=self.to_data_coor(event.x, event.y)
        btn,others=self._get_btn_and_others(event)
        self.tool.mouse_move(self, x, y, btn, **others)
        self.update()
        
    def on_mouse_down(self,event):
        obj, tol = self.get_obj_tol()
        x,y=self.to_data_coor(event.x, event.y)
        btn,others=self._get_btn_and_others(event)
        self.tool.mouse_down(self, x, y, btn, **others)
        self.update()
        
    def on_mouse_wheel(self,event):
        if event.data<0:
            wheel=-1
        else:
            wheel=1
        x,y=self.to_data_coor(px, py)
        btn,others=self._get_btn_and_others(event)
        self.tool.mouse_wheel(self, x, y, wheel, **others)
             
    def fit(self):
        self.update_box()
        oriw = self.oribox[2]-self.oribox[0]
        orih = self.oribox[3]-self.oribox[1]
        if not self.autofit: a,b,c,d = self.winbox
        else: 
            (a,b),(c,d) = (0,0), self.scrbox
            c, d = c*0.9, d*0.9
        if not self.ingrade:
            for i in self.scales[6::-1]:
                if oriw*i<c-a and orih*i<d-b: break
            self.scaidx = self.scales.index(i)
        else: i = min((c-a)*0.9/oriw, (d-b)*0.9/orih)
        self.zoom(i, 0, 0)
        lay(self.winbox, self.conbox)
        self.update()
        
    def draw_image(self, img, back, mode):
        out, bak, rgb = self.outimg, self.outbak, self.outrgb
        ori, cont = self.oribox, self.conbox
        cellbox = like(ori, cont, img.box)
        csbox = cross(self.winbox, cellbox)#取交叠部分【x1,y1,x2,y2】
        
        if min(csbox[2]-csbox[0], csbox[3]-csbox[1])<5: return
        shp = csbox[3]-csbox[1], csbox[2]-csbox[0]# 交叠部分的形状。
        o, m = mat(self.oribox, self.conbox, cellbox, csbox)# oribox是一个和旋转有关的量，conbox是承载图像的box
        # csbox是上面两个box重叠的部分，也是真正的绘图画布box。 winbox是盛放整个画布窗口的box。
        shp = tuple(np.array(shp).round().astype(np.int))
        
##        print('winbox',self.winbox)
##        print('conbox',self.conbox)
##        print('csbox',csbox,)
       
        if out is None or (out.shape, out.dtype) != (shp, img.dtype):
            self.outimg = np.zeros(shp, dtype=img.dtype)
        if not back is None and not back.img is None and (
            bak is None or (bak.shape, bak.dtype) != (shp, back.dtype)):
            self.outbak = np.zeros(shp, dtype=back.dtype)
        if rgb is None or rgb.shape[:2] != shp:
            self.outrgb = np.zeros(shp+(3,), dtype=np.uint8)
            self.outint = np.zeros(shp, dtype=np.uint8)
        if not back is None:
            mix_img(back.img, m, o, shp, self.outbak, 
                self.outrgb, self.outint, back.rg, back.lut,
                back.log, cns=back.cn, mode='set')
        
        mix_img(img.img, m, o, shp, self.outimg,
            self.outrgb, self.outint, img.rg, img.lut,
            img.log, cns=img.cn, mode=img.mode)
        self.draw_rgb(self.outrgb, *csbox[:2])
        
    def _update(self):
        super().update()
        
    def update(self, counter = [0,0]):
        self.update_box()
        if None in [self.winbox, self.conbox]: return
        if self.first:
            self.first = False
            return self.fit()
        counter[0] += 1
        start = time()
        
        for i in self.images: 
            self.draw_image(i, i.back, 0)
        
        counter[1] += time()-start
        if counter[0] == 50:
            print('frame rate:',int(50/max(0.001,counter[1])))
            counter[0] = counter[1] = 0

    def set_tool(self, tool:Tool):
        self.tool = tool
        
    def update_box(self):
        box = [1e10, 1e10, -1e10, -1e10]
        for i in self.images: box = merge(box, i.box)
        for i in self.marks.values(): box = merge(box, i.box)
        if box[2]<=box[0]: box[0], box[2] = box[0]-1e-3, box[2]+1e-3
        if box[1]<=box[3]: box[1], box[3] = box[1]-1e-3, box[3]+1e-3
        if self.winbox and self.oribox == box: return
        self.conbox = self.oribox = box   

        
    @property
    def scale(self):
        conw = self.conbox[2]-self.conbox[0]
        oriw = self.oribox[2]-self.oribox[0]
        conh = self.conbox[3]-self.conbox[1]
        orih = self.oribox[3]-self.oribox[1]
        l1, l2 = conw**2+conh**2, oriw**2+orih**2
        return l1**0.5 / l2**0.5


    def to_data_coor(self, x, y):
        if self.up: y = (self.winbox[3]-self.winbox[1]) - y
        x, y = x / self.scale, y / self.scale
        x += -self.conbox[0]/self.scale+self.oribox[0]
        y += -self.conbox[1]/self.scale+self.oribox[1]
        return x, y
        
    def to_panel_coor(self, x, y):
        x, y = x * self.scale, y * self.scale
        x += -self.oribox[0] * self.scale + self.conbox[0]
        y += -self.oribox[1] * self.scale + self.conbox[1]
        if self.up: y = (self.winbox[3]-self.winbox[1]) - y
        return x, y
                
    def on_paint(self, event):
        if self.buffer is None: return
        wx.BufferedPaintDC(self, self.buffer)
        
    def center(self, x, y, coord='win'):
        if coord=='data':
            x,y = self.to_panel_coor(x, y)
        dx = (self.winbox[2]-self.winbox[0])/2 - x
        dy = (self.winbox[3]-self.winbox[1])/2 - y
        for i,j in zip((0,1,2,3),(dx,dy,dx,dy)):
            self.conbox[i] += j
        lay(self.winbox, self.conbox)
    
    def move(self, dx, dy, coord='win'):
        if coord=='data':
            dx,dy = dx*self.scale, dy*self.scale
        arr = np.array(self.conbox)
        arr = arr.reshape((2,2))+(dx, dy)
        self.conbox = arr.ravel().tolist()
        self.update()
        
    def zoom(self, k, x, y, coord='win'):
        if coord=='data':
            x,y = self.to_panel_coor(x, y)
            if self.up: y = (self.winbox[3]-self.winbox[1]) - y
        box = np.array(self.conbox).reshape((2,2))
        box = (box - (x,y)) / self.scale * k + (x, y)
        self.conbox = box.ravel().tolist()
        if not self.autofit: return
        a,b,c,d = self.conbox
        if c-a<self.scrbox[0]*0.9 and d-b<self.scrbox[1]*0.9:
            print((c-a+4, d-b+4))
            self.config(width=c-a+4,height=d-b+4)#SetInitialSize((c-a+4, d-b+4))
        lay(self.winbox, self.conbox)
        #self.GetParent().Fit()
        
    def zoomout(self, x, y, coord='win', grade=True):
        if not self.ingrade:
            self.scaidx = min(self.scaidx + 1, len(self.scales)-1)
            i = self.scales[self.scaidx]
        else: i = self.scale * 1.5
        self.zoom(i, x, y, coord)
        self.update()

    def zoomin(self, x, y, coord='win'):
        if not self.ingrade:
            self.scaidx = max(self.scaidx - 1, 0)
            i = self.scales[self.scaidx]
        else: i = self.scale / 1.5
        self.zoom(i, x, y, coord)
        self.update()
        
    def on_size(self, event):
##        if max(self.GetClientSize())>20:
##            self.initBuffer()
        if len(self.images)+len(self.marks)==0: return
        
        self.winbox[2] = self.winfo_width()
        self.winbox[3] = self.winfo_height()
        self.update()
        
    def __del__(self):
        print('========== canvas del')
if __name__=='__main__':
# Create a window
    from sciapp.object import Image
    window = tk.Tk()
    window.title("OpenCV and Tkinter")

    # Load an image using OpenCV
    cv_img2 = cv2.cvtColor(cv2.imread("/home/hzy/Desktop/深度截图_选择区域_20200613232201.png"), cv2.COLOR_BGR2RGB)

     # Get the image dimensions (OpenCV stores image data as NumPy ndarray)
    # Create a canvas that can fit the above image
    canvas = Canvas(window)
    canvas.pack(side=tk.LEFT)
    image = Image()
    image.img = cv_img2
    image.pos = (100,200)
    image.cn = (0,1,2)
    canvas.images.append(image)
    ##c2= Canvas(window, width = width, height = height)
    ##c2.pack(side=tk.LEFT)
    # Use PIL (Pillow) to convert the NumPy ndarray to a PhotoImage

     
     # Add a PhotoImage to the Canvas

    window.mainloop()



import tkinter as tk
from tkinter import ttk
from scitk.canvas.mcanvas import MCanvas
from scitk.widgets import ToolBar, MenuBar, ParaDialog
from sciapp import App
from scitk.widgets.tkwidgetext.closablenotebook import ClosableNotebook
from scitk.widgets.toolbar import Toolbar
class CanvasFrame(ttk.Frame, App):
    def __init__(self, parent=None, autofit=False):
        App.__init__(self)
        ttk.Frame.__init__ (self, parent)
##        , id = wx.ID_ANY,
##                            title = 'CanvasFrame',
##                            pos = wx.DefaultPosition,
##                            size = wx.Size( 800, 600 ),
##                            style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )

        
        sizer = ttk.Frame()
        sizer.pack(expand = True, fill=tk.BOTH)
        self.canvas = MCanvas(sizer, autofit=autofit)
        self.canvas.pack(expand = True, fill=tk.BOTH)
        

        self.set_rg = self.canvas.set_rg
        self.set_lut = self.canvas.set_rg
        self.set_log = self.canvas.set_log
        self.set_mode = self.canvas.set_mode
        self.set_tool = self.canvas.set_tool
        self.set_imgs = self.canvas.set_imgs
        self.set_img = self.canvas.set_img
        self.set_cn = self.canvas.set_cn

##        self.Bind(wx.EVT_IDLE, self.on_idle)
##        self.Bind(wx.EVT_ACTIVATE, self.on_valid)
##        self.Bind(wx.EVT_CLOSE, self.on_close)

    def on_idle(self, event):
        if self.GetTitle()!=self.canvas.image.title:
            self.SetTitle(self.canvas.image.title)

    def set_title(self, ips): self.SetTitle(ips.title)

    def on_valid(self, event): pass

    def on_close(self, event): 
        event.Skip()

    def add_toolbar(self):
        toolbar = ToolBar(self)
        self.GetSizer().Insert(0, toolbar, 0, wx.EXPAND | wx.ALL, 0)
        return toolbar

    def add_menubar(self):
        menubar = MenuBar(self)
        self.SetMenuBar(menubar)
        return menubar

    def show_para(self, title, view, para, on_handle=None, on_ok=None, on_cancel=None, preview=False, modal=True):
        dialog = ParaDialog(self, title)
        dialog.init_view(view, para, preview, modal=modal, app=self)
        dialog.Bind('cancel', on_cancel)
        dialog.Bind('parameter', on_handle)
        dialog.Bind('commit', on_ok)
        return dialog.show()

class CanvasNoteBook(ClosableNotebook):
    def __init__(self, parent):
        ClosableNotebook.__init__( self, parent)
        self.pack(expand = True, fill = tk.BOTH)
##        self.Bind( wx.lib.agw.aui.EVT_AUINOTEBOOK_PAGE_CHANGED, self.on_valid) 
##        self.Bind( wx.lib.agw.aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.on_close)
##        self.Bind( wx.EVT_IDLE, self.on_idle)
##        self.SetArtProvider(aui.AuiSimpleTabArt())
        
    def on_idle(self, event):
        for i in range(self.GetPageCount()):
            title = self.GetPage(i).image.title
            if self.GetPageText(i) != title:
                self.SetPageText(i, title)

    def canvas(self, i=None):
        if not i is None: return self.GetPage(i)
        else: return self.GetCurrentPage()
        
    def set_background(self, img):
        self.GetAuiManager().SetArtProvider(ImgArtProvider(img))

    def add_canvas(self, mcanvas=None):
        if mcanvas is None: mcanvas = MCanvas(self)
        self.add_tab(mcanvas,text =  'Image')
        return mcanvas

    def set_title(self, panel, title):
        self.SetPageText(self.GetPageIndex(panel), title)

    def on_valid(self, event): pass

    def on_close(self, event): pass

class CanvasNoteFrame(ttk.Frame, App):
    def __init__(self, parent, title = 'CanvasNoteFrame'):
        App.__init__(self)
        ttk.Frame.__init__ ( self, parent, id = wx.ID_ANY,
                            title = title,
                            pos = wx.DefaultPosition,
                            size = wx.Size( 800, 600 ),
                            style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.notebook = CanvasNoteBook(self)
        self.canvas = self.notebook.canvas
        sizer.Add( self.notebook, 1, wx.EXPAND |wx.ALL, 0 )
        self.SetSizer(sizer)
        self.add_canvas = self.notebook.add_canvas
        self.Layout()
        self.Bind(wx.EVT_CLOSE, self.on_close)

    def add_toolbar(self):
        toolbar = ToolBar(self)
        self.GetSizer().Insert(0, toolbar, 0, wx.EXPAND | wx.ALL, 0)
        return toolbar 

    def add_menubar(self):
        menubar = MenuBar(self)
        self.SetMenuBar(menubar)
        return menubar
        
    def on_close(self, event):
        while self.notebook.GetPageCount()>0:
            self.notebook.DeletePage(0)
        event.Skip()
    
if __name__=='__main__':
    from skimage.data import camera, astronaut
    from skimage.io import imread

    
    app = tk.Tk()
    cf = CanvasFrame(None, autofit=False)
    cf.set_imgs([astronaut(), 255-astronaut()])
    cf.set_cn(0)
    cf.pack()
    app.mainloop()
    
    '''
    app = wx.App()
    cnf = CanvasNoteFrame(None)
    canvas = cnf.add_img()
    canvas.set_img(camera())

    canvas = cnf.add_img()
    canvas.set_img(camera())
    canvas.set_cn(0)
    
    cnf.Show()
    app.MainLoop()
    '''




from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as FigureCanvas
from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as NavigationToolbar

from matplotlib.figure import Figure

import tkinter as tk
from tkinter import ttk
import numpy as np
from scitk.widgets.tkwidgetext.closablenotebook import ClosableNotebook
class PlotCanvas(FigureCanvas):
    def __init__(self, parent, id=-1, fig=None, title='Plot'):
        self.figure =  Figure(figsize=(5,4),dpi=100)
        self.title = title
        super().__init__(self.figure,master=parent)

    def add_subplot(self, n=111, **key):
        return self.figure.add_subplot(n, **key)
        
class PlotFrame(ttk.Frame):
    def __init__(self, parent, toolbar=True):
        super().__init__(parent)

        self.figure = PlotCanvas(self)
        
        self.add_subplot = self.figure.add_subplot
        if toolbar: self.add_toolbar()
        self.figure._tkcanvas.pack(expand=True,fill=tk.BOTH)
        
        self.pack(fill=tk.BOTH,expand=True)

    def on_idle(self, event):
        if self.GetTitle()!=self.figure.title:
            self.SetTitle(self.figure.title)


    def add_toolbar(self):
        self.toolbar = NavigationToolbar(self.figure,self)
        self.toolbar.pack()

class PlotNoteBook(ClosableNotebook):
    def __init__(self, parent):
        super().__init__(parent)
        #self.pack(expand = True, fill = tk.BOTH)
        
    def on_idle(self, event):
        for i in range(self.GetPageCount()):
            title = self.GetPage(i).title
            if self.GetPageText(i) != title:
                self.SetPageText(i, title)

    def figure(self, i=None):
        return self.get_tab(i)
        
    def set_background(self, img):
        self.GetAuiManager().SetArtProvider(ImgArtProvider(img))

    def add_figure(self, figure=None):
        if figure is None: 
            return #grid =MGrid(self)
        self.add_tab(figure, text='hhhhhhhhhhhhhhhhhhhhhhhh' )
        return figure

    def set_title(self, panel, title):
        self.SetPageText(self.GetPageIndex(panel), title)

    def on_valid(self, event): pass

    def on_close(self, event): pass

    def mpl_connect(self, evt, method):
        if self.figure() is None: return
        self.figure().mpl_connect(
            'motion_notify_event', self.mouse_move)    



class PlotNoteFrame(ttk.Frame):
    def __init__(self, parent, toolbar=True):
        super().__init__ (parent)
        self.notebook = PlotNoteBook(self)
        self.figure = self.notebook.figure
        self.add_figure = self.notebook.add_figure
        self.notebook.pack(expand = True, fill = tk.BOTH)
        self.pack(expand = True, fill = tk.BOTH)
        
    def add_toolbar(self):
        self.toolbar = NavigationToolbar(self.notebook)
        self.toolbar.Realize()
        self.GetSizer().Add(self.toolbar, 0, wx.LEFT | wx.EXPAND)
        self.toolbar.update()
    
    def create_subplot(self):
        '''
        create a new subplot in a new tab, return a new subplot for plot.
        新建一个标签页，并且在其上创建一个新的subplot。返回值为plt.subplot。
        '''
        pframe = PlotFrame(self)
        fig = pframe.figure
        ax = fig.add_subplot()
        self.notebook.add_figure(pframe)    
        return ax
        
    def create_figure(self)->Figure:
        '''
        create a new figure in a new tab, return a new figure for plot.
        新建一个标签页，并且在其上创建一个新的figure。返回值为plt.figure。
        '''
        pframe = PlotFrame(self)
        fig = pframe.figure
        self.notebook.add_figure(pframe)    
        return fig
    
    def clear_figure(self,tab_id:int)->None:
        '''
        clear all plots on tab identified by tab_id. 
        清空tab_id所对应的标签页上的图像。
        '''
        f = self.get_figure(tab_id).clf()
    def get_figure_tab(self,tab_id:int)->PlotFrame:
        """获取tab_id对应的figure的标签页。"""
        return self.notebook.get_tab(tab_id)
        
    def get_figure(self,fig_id=None)->Figure:
        '''获取当前fig_id对应的Figure.'''
        return self.get_figure_tab(fig_id).figure.figure
        
    def delete_figure(self,tab_id:int):
        """
        delete figure tab identified by tab_id.
        删除tab_id表示的标签页及其上的图像。
        """
        self.notebook.delete_tab(tab_id)
    def update(self,tab_id=None):
        '''
        update figure identified by tab_id
        刷新tab_id对应的标签页的Figure，当清除重绘的时候应当调用此函数。
        '''
        canvas = self.get_figure_tab(0).figure
        canvas.draw()
        canvas.flush_events()
        
if __name__ == '__main__':
    app = tk.Tk()
    pnf=PlotNoteFrame(app)
    pnb=pnf.notebook
    pframe = PlotFrame(pnb)
    ax = pframe.add_subplot(111)
    x = np.linspace(0,10,100)
    y = np.sin(x)
    ax.plot(x, y)
    ax.grid()
    ax.set_title('abc')
    pnb.add_figure(pframe)
    
    pframe2 = PlotFrame(pnb)
    ax = pframe2.add_subplot()
    x = np.linspace(0,10,100)
    y = np.exp(x)
    ax.plot(x, y)
    ax.grid()
    ax.set_title('def')
    pnb.add_figure(pframe2)
    
    app.mainloop()





import tkinter as tk
from tkinter import ttk

def hot_key(txt):
    sep = txt.split('-')
    acc, code = wx.ACCEL_NORMAL, -1
    if 'Ctrl' in sep: acc|= wx.ACCEL_CTRL
    if 'Alt' in sep: acc|= wx.ACCEL_ALT
    if 'Shift' in sep: acc|= wx.ACCEL_SHIFT
    fs = ['F%d'%i for i in range(1,13)]
    if sep[-1] in fs:
        code = 340+fs.index(sep[-1])
    elif len(sep[-1])==1: code = ord(sep[-1])
    return acc, code

class MenuBar(ttk.Frame):
    def __init__(self, app):
        wx.MenuBar.__init__(self)
        self.app = app
        app.SetMenuBar(self)

    def parse(self, ks, vs, pt, short, rst):
        if isinstance(vs, list):
            menu = wx.Menu()
            for kv in vs:
                if kv == '-': menu.AppendSeparator()
                else: self.parse(*kv, menu, short, rst)
            pt.Append(1, ks, menu)
        else:
            item = wx.MenuItem(pt, -1, ks)
            if ks in short:
                rst.append((short[ks], item.GetId()))
            f = lambda e, p=vs: p().start(self.app)
            self.Bind(wx.EVT_MENU, f, item)
            pt.Append(item)

    def Append(self, id, item, menu):
        wx.MenuBar.Append(self, menu, item)
        
    def load(self, data, shortcut={}):
        rst = []
        for k,v in data[1]: 
            self.parse(k, v, self, shortcut, rst)
        rst = [(*hot_key(i[0]), i[1]) for i in rst]
        return wx.AcceleratorTable(rst)

    def on_menu(self, event): print('here')

    def clear(self):
        while self.GetMenuCount()>0: self.Remove(0)
        

if __name__ == '__main__':
    class P:
        def __init__(self, name):
            self.name = name

        def start(self):
            print(self.name)

        def __call__(self):
            return self
        
    data = ('menu', [
            ('File', [('Open', P('O')),
                      '-',
                      ('Close', P('C'))]),
            ('Edit', [('Copy', P('C')),
                      ('A', [('B', P('B')),
                             ('C', P('C'))]),
                      ('Paste', P('P'))])])
    
    app = wx.App()
    frame = wx.Frame(None)
    menubar = MenuBar()
    menubar.load(data)
    frame.SetMenuBar(menubar)
    frame.Show()
    app.MainLoop()




from sciapp.action import ImgAction
from scipy.ndimage import gaussian_filter

class Gaussian(ImgAction):
    name = 'Gaussian'
    note = ['auto_snap', 'preview']
    para = {'sigma':2}
    view = [(float, 'sigma', (0, 30), 1, 'sigma', 'pix')]

    def run(self, ips, img, snap, para):
        gaussian_filter(snap, para['sigma'], output=img)

class Undo(ImgAction):
    name = 'Undo'
    def run(self, ips, img, snap, para): ips.swap()


import tkinter as tk
from tkinter import ttk
from scitk.widgets.tkwidgetext.scrolledcanvas import ScrolledCanvas
def parse(cont):
    ls = cont.split('\n')
    workflow = {'title':ls[0], 'chapter':[]}
    for line in ls[2:]:
        line = line.strip()
        if line == '':continue
        if line.startswith('## '):
            chapter = {'title':line[3:], 'section':[]}
            workflow['chapter'].append(chapter)
        elif line[1:3] == '. ':
            section = {'title':line[3:]}
        else:
            section['hint'] = line
            chapter['section'].append(section)
    print(workflow)
    return workflow

class WorkFlowPanel (ttk.Frame):
    def __init__( self, parent ):
        super().__init__ (parent)
        self.app, self.f = parent, print

    def SetValue(self, cont):
        self.workflow, self.cont = parse(cont), cont
        self.workflow_canvas = ScrolledCanvas(self, orient = 'horizontal')
        self.scr_workflow = self.workflow_canvas.scrollable_frame
        
        self.workflow_canvas .pack(side=tk.LEFT, expand = True, fill = tk.BOTH)
        
        for chapter in self.workflow['chapter']:
            self.pan_chapter = ttk.Frame(self.scr_workflow)
            
            
            b=tk.Button(self.pan_chapter,text='ddddddddddd')
            b.pack()

            self.pan_chapter.pack(expand=True,fill=tk.Y,side=tk.LEFT) 
            
            self.lab_chapter = ttk.Label(self.pan_chapter, text = chapter['title'])
            self.lab_chapter.pack()
            
            for section in chapter['section']:
                btn = ttk.Button( self.pan_chapter, text= section['title'])
                btn.pack()
                btn.config(command = lambda x=section['title']: self.f(x))
                btn.bind( '<Motion>', lambda e, info=section['hint']: self.set_info(info))# show information when mouse is moving
           
            self.btn_snap = ttk.Label( self.pan_chapter, text= u" Snap ")
            self.btn_snap.pack()
            
            self.btn_load = ttk.Button( self.pan_chapter, text = u" Load ")
            self.btn_load.pack()
            
            self.btn_step = ttk.Button( self.pan_chapter, text= u" >> ")
            self.btn_step.pack()

        self.btn_help = ttk.Button( self, text = u" Click For Detail Document ", 
                                   command=lambda event='e'  : self.on_help(event)) # 'e'  is just an non_sense argument for occupy args position.
        self.btn_help.pack(side=tk.LEFT)
        
        self.txt_info = tk.Text( self )
        self.txt_info.pack(side=tk.LEFT)

    def set_info(self, info):
        self.txt_info.delete('0.0',tk.END)
        self.txt_info.insert('0.0',info)
    
    def f(self,event):
        self.on_spn(event)
    def on_spn(self, event):
        v = self.spn_scroll.GetValue()
        self.scr_workflow.Scroll(v, 0)
        self.spn_scroll.SetValue(self.scr_workflow.GetViewStart()[0])

    def Bind(self, event, f=print): self.f = f

    def on_help(self, event):
        self.app.show_md(self.cont, self.workflow['title'])

if __name__ == '__main__':

    cont = '''Title
=====
## Chapter1
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
## Chapter2
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
## Chapter2
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
## Chapter2
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
## Chapter2
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
## Chapter2
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
## Chapter2
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
## Chapter2
1. Section1
some coment for section1 ...
2. Section2
some coment for section2 ...
'''

    app = tk.Tk()
    frame = ttk.Frame(app)
##    sizer = wx.BoxSizer(wx.VERTICAL)
    wf = WorkFlowPanel(app)
    wf.SetValue(cont*1)
    wf.pack(expand=1,fill=tk.X)
    frame.pack(expand=1,fill=tk.BOTH)
    btn2 = ttk.Button(frame, text='sssssssssss')
    btn2.pack(expand=1 , fill=tk.BOTH)
    app.mainloop()



import sys

sys.path.append('/media/hzy/程序/novalide/forgitcommit/scitk/scitk/')
import tkinter as tk
from tkinter import ttk
import os
import sys
import time
import threading

from scitk.grid import GridNoteBook
from scitk.widgets import normal
from scitk.widgets import paradialog
from scitk.widgets.controls import ButtonWidget,SwitchWidget
from scitk.plot.plot import PlotFrame, PlotNoteFrame
import scitk

import numpy as np

view_dic = {'plot': PlotNoteFrame}
control_dic = {'button':ButtonWidget,'switch':SwitchWidget}


class ParamWidget():
    '''
    parent
    widget_name
    widget_type
    widget_args
    callback: function call when value is changed.
    widget_text
    '''

    def __init__(self, parent: ttk.Frame, widget_name: str, widget_type: str, widget_args: tuple, callback=None,
                 widget_text='', initial_value=None):
        self.name = widget_name

        if widget_type in paradialog.widgets.keys():
            self.widget = paradialog.widgets[widget_type](parent, *widget_args)
        else:
            raise Exception("Widget type \'%s\' is invalid." % widget_type)

        self.widget.pack()
        self.callback = None
        self.widget.on_para_change = callback
        if initial_value is not None:
            self.widget.SetValue(initial_value)
        pass

    def get_value(self):
        return self.widget.GetValue()

    def set_value(self, val):
        self.widget.SetValue(val)



class SciApp(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)

        menubar = tk.Menu(self.master)
        self.master.config(menu=menubar)

        ##        fileMenu = tk.Menu(menubar)
        ##        fileMenu.add_command(label="Exit", command=lambda:print('hhhhh'))
        ##        menubar.add_cascade(label="File", menu=fileMenu)

        self.toolbar_frame = ttk.Frame(self)
        self.toolbar_frame.pack(fill=tk.X)
        self.toolbar = scitk.widgets.toolbar.Toolbar(self.toolbar_frame)
        self.toolbar.pack(fill=tk.X, expand=1)
        # self.toolbar .add_tool('/media/hzy/程序/novalide/forgitcommit/NovalIDE/noval/bmp_source/unittest_wizard.png',lambda:print('ddd'))

        ##        b = ttk.Button(self.toolbar_frame,text='aaaaa')
        ##        b.pack(side=tk.LEFT)

        frame1 = ttk.Frame(self)
        frame1.pack(expand=1, fill=tk.BOTH)

        self.toolbar_left = ttk.Frame(frame1)
        self.toolbar_left.pack(side=tk.LEFT, fill=tk.Y, expand=True)

        self.mainPane = ttk.Frame(frame1)
        self.mainPane.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        self.m1 = tk.PanedWindow(self.mainPane)  # ,showhandle =False, sashrelief="sunken")
        self.m1.pack(fill="both", expand=1)

        left = ttk.Frame(self.m1)
        self.m1.add(left)

        self.m2 = tk.PanedWindow(self.m1, orient="vertical")  # ,showhandle=True, sashrelief="sunken")
        self.m1.add(self.m2)

        self.view_panel = ttk.Frame(self.m2)
        self.m2.add(self.view_panel)

        self.sub_panel = ttk.Frame(self.m2)
        self.m2.add(self.sub_panel)

        self.control_panel = ttk.Frame(self.m1)
        self.m1.add(self.control_panel)

        self.control_widget_dict = {}
        self.view_widget_dict = {}
        self.param_widget_dict = {}

    def add_view(self, view_type, view_name):
        v = view_dic[view_type](self.view_panel)
        v.pack(fill=tk.BOTH, expand=True)
        self.view_widget_dict[view_name] = v

    def add_param(self, name: str, param_type: object, arguments: tuple, callback=None, initial_value=None):
        '''
        name:str
        param_type
        arguments
        callback
        initial_value
        '''
        self.param_widget_dict[name] = ParamWidget(self.control_panel, name, param_type, arguments, callback=callback,
                                                   initial_value=initial_value)
        self.param_widget_dict[name].widget.set_app(self)

    def add_control(self, name='', control_type='',callback='', text=''):
        '''
        添加控件。
        name:
        control_type
        callback
        text
        '''
        if name in self.control_widget_dict.keys():
            raise  Exception("Control widget '%s' is already defined"%name)
        if control_type not in control_dic.keys():
            raise  Exception("Control Type '%s' is invalid."%control_type)
        self.control_widget_dict[name] = control_dic[control_type](self.control_panel, widget_name=name,
                            callback=lambda: callback(self), widget_text=text)

    def get_param_widget(self, name) -> ParamWidget:
        if name not in self.param_widget_dict.keys():
            raise Exception('Param widget name \'%s\' is not defined!' % name)
        else:
            return self.param_widget_dict[name]

    def get_control_widget(self, name: str) -> ButtonWidget:

        if name not in self.control_widget_dict.keys():
            raise ('Control widget name \'%s\' is not defined!' % name)
        else:
            return self.control_widget_dict[name]

    def get_view_widget(self, button_name: str):

        if button_name not in self.view_widget_dict.keys():
            raise (Exception('View widget name \'%s\' is not defined!' % button_name))
        else:
            return self.view_widget_dict[button_name]


def run_app(add_widgets_fcn):
    app = tk.Tk()
    sa = SciApp(app)

    sa.pack(expand=True, fill=tk.BOTH)
    add_widgets_fcn(sa)
    print('Finished loading widgets. widgets are as follows:')
    print('view_widgets:',list(sa.view_widget_dict.keys()))
    print('param_widgets:',list(sa.param_widget_dict.keys()))
    print('control_widgets:',list(sa.control_widget_dict.keys()))
    app.geometry('800x600+100+100')
    sa.mainloop()


if __name__ == '__main__':
    run_app()



from tkinter import ttk
import tkinter as tk   
from scitk.widgets.tkwidgetext.closablenotebook import ClosableNotebook
from scitk.widgets.toolbar import Toolbar

class MGrid(ttk.Frame):
    def __init__(self, parent=None, autofit=False):
        ttk.Frame.__init__ ( self, parent)
##        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
##        
##        self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )
        sizer = ttk.Frame(self)# wx.BoxSizer( wx.VERTICAL )
        sizer.pack(expand = True, fill=tk.BOTH)
        self.lab_info =ttk.Label(sizer,text ='information')
        self.lab_info.pack()
        
##        self.Bind(wx.EVT_IDLE, self.on_idle)
##        sizer.Add( self.lab_info, 0, wx.EXPAND, 0 )
##        
        self.grid = Grid(self)
        self.grid.pack(fill=tk.BOTH,expand=True)
##        sizer.Add( self.grid, 1, wx.EXPAND |wx.ALL, 0 )
##        self.SetSizer(sizer)
        self.select = self.grid.select
        self.set_data = self.grid.set_data

    @property
    def table(self): return self.grid.table

    @property
    def name(self): return self.grid.table.name

    def on_idle(self, event):
        if self.table.data is None: return
        if self.lab_info.GetLabel() != self.table.info:
            self.lab_info.SetLabel(self.table.info)

class GridFrame(ttk.Frame):
    def __init__(self, parent=None):
        ttk.Frame.__init__ ( self, parent)
##                            , id = wx.ID_ANY,
##                            title = 'GridFrame',
##                            pos = wx.DefaultPosition,
##                            size = wx.Size( 800, 600 ),
##                            style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
        sizer = ttk.Frame(self)#wx.BoxSizer(wx.VERTICAL)
        sizer.pack(expand=True,fill=tk.BOTH)
        self.grid = MGrid(sizer)
##        self.SetSizer(sizer)
        self.set_data = self.grid.set_data
##        self.Bind(wx.EVT_IDLE, self.on_idle)

    def on_idle(self, event):
        if self.GetTitle()!=self.grid.table.name:
            self.SetTitle(self.grid.table.name)
    
    def set_title(self, tab): self.SetTitle(tab.name)

    def on_valid(self, event): event.Skip()

    def on_close(self, event): event.Skip()

    def Show(self):
        self.Fit()
        wx.Frame.Show(self)

class GridNoteBook(ClosableNotebook):
    def __init__(self, parent):
        super().__init__(parent)
        self.pack(expand = True, fill = tk.BOTH)
        
    def on_idle(self, event):
        for i in range(self.GetPageCount()):
            title = self.GetPage(i).table.title
            if self.GetPageText(i) != title:
                self.SetPageText(i, title)

    def grid(self, i=None)->ttk.Frame:
        return self.get_tab(i)
        
    def set_background(self, img):
        self.GetAuiManager().SetArtProvider(ImgArtProvider(img))

    def add_grid(self, grid=None):
        if grid is None: 
            return #grid =MGrid(self)
        self.add_tab(grid, text='hhhhhhhhhhhhhhhhhhhhhhhh' )
        return grid

    def set_title(self, panel, title):
        self.SetPageText(self.GetPageIndex(panel), title)

    def on_valid(self, event): pass

    def on_close(self, event): pass
    

class GridNoteFrame(ttk.Frame):
    def __init__(self, parent):
        super().__init__ (parent)
        
        
        self.notebook = GridNoteBook(self)
        self.toolbarFrame=ttk.Frame(self)
        self.grid = self.notebook.grid # 这个是重写了函数
        self.toolbarFrame.pack(expand=True,fill=tk.X)
        self.notebook.pack(expand=True,fill=tk.BOTH)
        self.add_grid = self.notebook.add_grid

        self.pack(expand=True,fill=tk.BOTH)

    def add_toolbar(self):
        toolbar = ToolBar(self.toolbarFrame)
        toolbar.pack()
        return toolbar 

    def on_close(self, event):
        while self.notebook.GetPageCount()>0:
            self.notebook.DeletePage(0)
        event.Skip()

if __name__=='__main__':
    from scitk.grid.grid import Grid
    import pandas as pd
    import numpy as np
    window = tk.Tk()
    gnf=GridNoteFrame(window)
    nb=gnf.notebook
    grid2=Grid(nb)
    nb.add_grid(Grid(nb))
    nb.add_grid(grid2)
    nb.add_grid(Grid(nb))
    nb.add_grid(Grid(nb))
    nb.delete_tab('current')
    nb.grid(1).set_data(pd.DataFrame(np.random.randn(100,5)*100))
    window.mainloop()



from ..tolact import ShapeTool
from sciapp.object import *
from sciapp.util import geom_union
from numpy.linalg import norm
import numpy as np

def mark(shp, types = 'all'):
	pts = []
	if not (types=='all' or shp.dtype in types): return pts
	if shp.dtype == 'point':
		pts.append([shp.body])
	if shp.dtype == 'points':
		pts.append(shp.body)
	if shp.dtype == 'line':
		pts.append(shp.body)
	if shp.dtype == 'lines':
		pts.extend(shp.body)
	if shp.dtype == 'polygon' and len(shp.body)==1:
		pts.append(shp.body[0])
	if shp.dtype == 'polygons':
		for i in shp.body:
			if len(i) != 1: continue
			pts.append(i[0])
	if shp.dtype == 'rectangle':
		l,t,w,h = shp.body
		ps = np.mgrid[l:l+w:3j, t:t+h:3j].T.reshape((-1,2))
		pts.append(ps)
	if shp.dtype == 'rectangles':
		for i in range(len(shp.body)):
			l,t,w,h = shp.body[i]
			ps = np.mgrid[l:l+w:3j, t:t+h:3j].T.reshape((-1,2))
			pts.append(ps)
	if shp.dtype == 'ellipse':
			x0, y0, l1, l2, ang = shp.body
			mat = np.array([[np.cos(-ang),-np.sin(-ang)],
						  [np.sin(-ang),np.cos(-ang)]])
			ps = np.mgrid[-l1:l1:3j, -l2:l2:3j].T.reshape((-1,2))
			pts.append(mat.dot(ps.T).T + (x0, y0))
	if shp.dtype == 'ellipses':
		for i in range(len(shp.body)):
			x0, y0, l1, l2, ang = shp.body[i]
			mat = np.array([[np.cos(-ang),-np.sin(-ang)],
						  [np.sin(-ang),np.cos(-ang)]])
			ps = np.mgrid[-l1:l1:3j, -l2:l2:3j].T.reshape((-1,2))
			pts.append(mat.dot(ps.T).T + (x0, y0))
	if shp.dtype == 'layer':
		minl, obj = 1e8, None
		for i in shp.body:
			pts.extend(mark(i, types))
	return pts

def pick_obj(shp, x, y, lim, types='all'):
	obj, minl = None, lim
	if not (types=='all' or shp.dtype in types): 
		return m, obj, minl
	if shp.dtype == 'layer':
		for i in shp.body:
			o, l = pick_obj(i, x, y, lim, types)
			if l < minl: 
				obj, minl = o, l
	elif shp.dtype in 'polygons':
		b = shp.to_geom().contains(Point([x, y]).to_geom())
		if b : return shp, 0
	else:
		d = shp.to_geom().distance(Point([x, y]).to_geom())
		if d<minl: obj, minl = shp, d
	return obj, minl

def pick_point(shp, x, y, lim, types='all'):
	m, obj, minl = None, None, lim
	if not (types=='all' or shp.dtype in types): 
		return m, obj, minl
	if shp.dtype == 'point':
		l = ((shp.body-(x, y))**2).sum()
		if l < minl: 
			m, obj, minl = shp, shp.body, l
	if shp.dtype == 'points':
		l = norm(shp.body-(x,y), axis=1)
		n = np.argmin(l)
		l = l[n]
		if l < minl: 
			m, obj, minl = shp, shp.body[n], l
	if shp.dtype == 'line':
		l = norm(shp.body-(x,y), axis=1)
		n = np.argmin(l)
		l = l[n]
		if l < minl: 
			m, obj, minl = shp, shp.body[n], l
	if shp.dtype == 'lines':
		for line in shp.body:
			l = norm(line-(x,y), axis=1)
			n = np.argmin(l)
			l = l[n]
			if l < minl: 
				m, obj, minl = shp, line[n], l
	if shp.dtype == 'polygon' and len(shp.body)==1:
		l = norm(shp.body[0]-(x,y), axis=1)
		n = np.argmin(l)
		l = l[n]
		if l < minl: 
			m, obj, minl = shp, shp.body[0][n], l
	if shp.dtype == 'polygons':
		for i in shp.body:
			if len(i) != 1: continue
			l = norm(i[0]-(x,y), axis=1)
			n = np.argmin(l)
			l = l[n]
			if l < minl: 
				m, obj, minl = shp, i[0][n], l
	if shp.dtype == 'rectangle':
		l,t,w,h = shp.body
		pts = np.mgrid[l:l+w:3j, t:t+h:3j].T.reshape((-1,2))
		names = ['lt','t','rt','l','o','r','lb','b','rb']
		l = norm(pts-(x,y), axis=1)
		n = np.argmin(l)
		if l[n] < minl:
			m, obj, minl = shp, names[n], l[n]
	if shp.dtype == 'rectangles':
		for i in range(len(shp.body)):
			l,t,w,h = shp.body[i]
			pts = np.mgrid[l:l+w:3j, t:t+h:3j].T.reshape((-1,2))
			names = ['lt','t','rt','l','o','r','lb','b','rb']
			l = norm(pts-(x,y), axis=1)
			n = np.argmin(l)
			if l[n] < minl:
				m, obj, minl = shp, (names[n], i), l[n]
	if shp.dtype == 'ellipse':
			x0, y0, l1, l2, ang = shp.body
			mat = np.array([[np.cos(-ang),-np.sin(-ang)],
						  [np.sin(-ang),np.cos(-ang)]])
			pts = np.mgrid[-l1:l1:3j, -l2:l2:3j].T.reshape((-1,2))
			pts = mat.dot(pts.T).T + (x0, y0)
			names = ['lt','t','rt','l','o','r','lb','b','rb']
			l = norm(pts-(x,y), axis=1)
			n = np.argmin(l)
			if l[n] < minl:
				m, obj, minl = shp, names[n], l[n]
	if shp.dtype == 'ellipses':
		for i in range(len(shp.body)):
			x0, y0, l1, l2, ang = shp.body[i]
			mat = np.array([[np.cos(-ang),-np.sin(-ang)],
						  [np.sin(-ang),np.cos(-ang)]])
			pts = np.mgrid[-l1:l1:3j, -l2:l2:3j].T.reshape((-1,2))
			pts = mat.dot(pts.T).T + (x0, y0)
			names = ['lt','t','rt','l','o','r','lb','b','rb']
			l = norm(pts-(x,y), axis=1)
			n = np.argmin(l)
			if l[n] < minl:
				m, obj, minl = shp, (names[n], i), l[n]
	if shp.dtype == 'layer':
		# minl, obj = 1e8, None
		for i in shp.body:
			h, o, l = pick_point(i, x, y, lim, types)
			if l < minl: 
				m, obj, minl = h, o, l
	return m, obj, minl

def drag(shp, pt, x, y, types='all'):
	if not (types=='all' or shp.dtype in types): return
	if shp.dtype == 'rectangle':
		body = shp.body
		if pt == 'o':body[:2] = (x, y) - body[2:]/2
		if 'l' in pt:body[[0,2]] = x, body[0]+body[2]-x
		if 'r' in pt:body[2] = x - body[0]
		if 't' in pt:body[[1,3]] = y, body[1]+body[3]-y
		if 'b' in pt:body[3] = y - body[1]
	elif shp.dtype == 'rectangles':
		pt, i = pt
		body = shp.body[i]
		if pt == 'o':body[:2] = (x, y) - body[2:]/2
		if 'l' in pt:body[[0,2]] = x, body[0]+body[2]-x
		if 'r' in pt:body[2] = x - body[0]
		if 't' in pt:body[[1,3]] = y, body[1]+body[3]-y
		if 'b' in pt:body[3] = y - body[1]
	elif shp.dtype == 'ellipse':
		if pt == 'o': 
			shp.body[:2] = x, y
			return
		x0, y0, l1, l2, ang = shp.body
		v1, v2 = (np.array([[np.cos(-ang),-np.sin(-ang)],
			[np.sin(-ang),np.cos(-ang)]]) * (l1, l2)).T
		l, r, t, b = np.array([-v1, v1, -v2, v2]) + (x0, y0)
		if 'l' in pt: l = v1.dot([x-x0, y-y0])*v1/l1**2+(x0, y0)
		if 'r' in pt: r = v1.dot([x-x0, y-y0])*v1/l1**2+(x0, y0)
		if 't' in pt: t = v2.dot([x-x0, y-y0])*v2/l2**2+(x0, y0)
		if 'b' in pt: b = v2.dot([x-x0, y-y0])*v2/l2**2+(x0, y0)
		k = np.linalg.inv(np.array([-v2,v1]).T).dot((l+r-t-b)/2)
		shp.body[:2] = (l+r)/2 + v2*k[0]
		shp.body[2:4] = np.dot(r-l, v1)/l1/2, np.dot(b-t, v2)/l2/2
	elif shp.dtype == 'ellipses':
		pt, i = pt
		body = shp.body[i]
		if pt == 'o': 
			body[:2] = x, y
			return
		x0, y0, l1, l2, ang = body
		v1, v2 = (np.array([[np.cos(-ang),-np.sin(-ang)],
			[np.sin(-ang),np.cos(-ang)]]) * (l1, l2)).T
		l, r, t, b = np.array([-v1, v1, -v2, v2]) + (x0, y0)
		if 'l' in pt: l = v1.dot([x-x0, y-y0])*v1/l1**2+(x0, y0)
		if 'r' in pt: r = v1.dot([x-x0, y-y0])*v1/l1**2+(x0, y0)
		if 't' in pt: t = v2.dot([x-x0, y-y0])*v2/l2**2+(x0, y0)
		if 'b' in pt: b = v2.dot([x-x0, y-y0])*v2/l2**2+(x0, y0)
		k = np.linalg.inv(np.array([-v2,v1]).T).dot((l+r-t-b)/2)
		body[:2] = (l+r)/2 + v2*k[0]
		body[2:4] = np.dot(r-l, v1)/l1/2, np.dot(b-t, v2)/l2/2
	else: pt[:] = x, y

def offset(shp, dx, dy):
	if shp.dtype in {'rectangle', 'ellipse', 'circle'}:
		shp.body[:2] += dx, dy
	elif shp.dtype in {'rectangles', 'ellipses', 'circles'}:
		shp.body[:,:2] += dx, dy
	elif isinstance(shp, np.ndarray):
		shp += dx, dy
	elif isinstance(shp.body, list):
		for i in shp.body: offset(i, dx, dy)

class BaseEditor(ShapeTool):
	def __init__(self, dtype='all'):
		self.status, self.oldxy, self.p = '', None, None
		self.pick_m, self.pick_obj = None, None

	def mouse_down(self, shp, x, y, btn, **key):
		self.p = x, y
		if btn==2:
			self.status = 'move'
			self.oldxy = key['px'], key['py']
		if btn==1 and self.status=='pick':
			m, obj, l = pick_point(shp, x, y, 5)
			self.pick_m, self.pick_obj = m, obj
		if btn==1 and self.pick_m is None:
			m, l = pick_obj(shp, x, y, 5)
			self.pick_m, self.pick_obj = m, None
		if btn==3:
			obj, l = pick_obj(shp, x, y, 5)
			if key['alt'] and not key['ctrl']:
				if obj is None: del shp.body[:]
				else: shp.body.remove(obj)
				shp.dirty = True
			if key['shift'] and not key['alt'] and not key['ctrl']:
				layer = geom2shp(geom_union(shp.to_geom()))
				shp.body = layer.body
				shp.dirty = True
			if not (key['shift'] or key['alt'] or key['ctrl']):
				key['canvas'].fit()

	def mouse_up(self, shp, x, y, btn, **key):
		self.status = ''
		if btn==1:
			self.pick_m = self.pick_obj = None
			if not (key['alt'] and key['ctrl']): return
			pts = mark(shp)
			if len(pts)>0: 
				pts = Points(np.vstack(pts), color=(255,0,0))
				key['canvas'].marks['anchor'] = pts
			shp.dirty = True

	def mouse_move(self, shp, x, y, btn, **key):
		self.cursor = 'arrow'
		if self.status == 'move':
			ox, oy = self.oldxy
			up = (1,-1)[key['canvas'].up]
			key['canvas'].move(key['px']-ox, (key['py']-oy)*up)
			self.oldxy = key['px'], key['py']
		if key['alt'] and key['ctrl']:
			self.status = 'pick'
			if not 'anchor' in key['canvas'].marks: 
				pts = mark(shp)
				if len(pts)>0: 
					pts = Points(np.vstack(pts), color=(255,0,0))
					key['canvas'].marks['anchor'] = pts
			if 'anchor' in key['canvas'].marks:
				m, obj, l = pick_point(key['canvas'].marks['anchor'], x, y, 5)
				if not m is None: self.cursor = 'hand'
		elif 'anchor' in key['canvas'].marks: 
			self.status = ''
			del key['canvas'].marks['anchor']
			shp.dirty = True
		if not self.pick_obj is None and not self.pick_m is None:
			drag(self.pick_m, self.pick_obj, x, y)
			pts = mark(self.pick_m)
			if len(pts)>0:
				pts = np.vstack(pts)
				key['canvas'].marks['anchor'] = Points(pts, color=(255,0,0))
			self.pick_m.dirty = True
			shp.dirty = True
		if self.pick_obj is None and not self.pick_m is None:
			offset(self.pick_m, x-self.p[0], y-self.p[1])
			pts = mark(self.pick_m)
			if len(pts)>0:
				pts = np.vstack(pts)
				key['canvas'].marks['anchor'] = Points(pts, color=(255,0,0))
			self.p = x, y
			self.pick_m.dirty =shp.dirty = True

	def mouse_wheel(self, shp, x, y, d, **key):
		if d>0: key['canvas'].zoomout(x, y, coord='data')
		if d<0: key['canvas'].zoomin(x, y, coord='data')


from setuptools import setup,find_packages
setup(name='scitk',
      version='0.0.2',
      description='python GUI framework like sciwx,  based on tkinter, pillow(PIL) and various of functional libs',
      url='https://gitee.com/hzy15610046011/scitk',
      author='Zhanyi Hou',
      author_email='3120388018@qq.com',
      license='MuLan2.0',
      packages=find_packages(),
      include_package_data = True,
      install_requires = ['pillow']
      )#package_data={'':['hat.png']})
     # zip_safe=False,install_requires = ['pyqt5','pyqtgraph','numpy','pyopengl'])



import tkinter as tk
from tkinter import ttk
import pandas as pd
import numpy as np

from scitk.widgets.scrollabletreeviewframe import ScrollableTreeviewFrame
class Grid(ScrollableTreeviewFrame):
    def __init__(self,master):
        super().__init__(master)

    def set_data(self,data):
        columns = ['']+list(data.columns.values)
        
        self.set_widgets(columns)
        self.treeview.bind('<ButtonRelease-1>', self.treeview_click)
        for col in columns:
            self.treeview.column(col, anchor = "center")
            self.treeview.heading(col, text = col)

        for i in range(data.shape[0]):
            row=data.iloc[i]
            #print(row[0],list(row))
            self.treeview.insert('', i, values = [i]+list(row))
            i += 1
         
    def treeview_click(event):  # 单击
        for item in self.tree.selection():
            item_text = self.tree.item(item, "values")
            print(item_text)
 
    def selectTree(event):
        for item in tree.selection():
            item_text = tree.item(item, "values")
            print(item_text)

if __name__ == '__main__':
    window = tk.Tk()
    # 设置窗口大小
    winWidth = 600
    winHeight = 400
    # 获取屏幕分辨率
    screenWidth = window.winfo_screenwidth()
    screenHeight = window.winfo_screenheight()
     
    x = int((screenWidth - winWidth) / 2)
    y = int((screenHeight - winHeight) / 2)
     
    # 设置主窗口标题
    window.title("TreeView参数说明")
    # 设置窗口初始位置在屏幕居中
    window.geometry("%sx%s+%s+%s" % (winWidth, winHeight, x, y))
    # 设置窗口图标

    # 设置窗口宽高固定
    window.resizable(0, 0)
     
    # 定义列的名称


    grid=Grid(window)
    
    # 选中行
    #tree.bind('<<TreeviewSelect>>', selectTree)
     
    window.mainloop()



import tkinter as tk
from tkinter import ttk
import os
import sys
import time
import threading


from scitk.grid import GridNoteBook

import scitk.app
class SciApp(tk.Frame):
    def __init__(self,parent):
        tk.Frame.__init__(self,parent)
        self.m1 = tk.PanedWindow(self)
        self.m1.pack(fill="both", expand=1)
         
        left = tk.Label(self.m1, text="left pane")
        self.m1.add(left)
         
        self. m2 = tk.PanedWindow(self,orient="vertical",showhandle=True, sashrelief="sunken")
        self.m1.add(self.m2)
         
        top = tk.Label(self.m2, text="top pane")
        self.m2.add(top)
         
        bottom = tk.Label(self.m2, text="bottom pane")
        self.m2.add(bottom)

if __name__=='__main__':
    app=tk.Tk()
    sa=SciApp(app)
    sa.pack(expand=True,fill=tk.BOTH)
    app.geometry('1024x768+100+100')
    sa.mainloop()



# -*- coding: utf-8 -*-
"""
Created on Fri Dec  2 23:48:33 2016
@author: yxl
"""

import threading
import numpy as np
from time import time, sleep

def process_channels(plg, ips, src, des, para):
    if ips.channels>1 and not 'not_channel' in plg.note:
        for i in range(ips.channels):
            rst = plg.run(ips, src if src is None else src[:,:,i], des[:,:,i], para)
            if not rst is des and not rst is None:
                des[:,:,i] = rst
    else:
        rst = plg.run(ips, src, des, para)
        if not rst is des and not rst is None:
            des[:] = rst
    return des

def process_one(plg, ips, src, img, para, callafter=None):
    if callafter != 'no record':
        plg.app.record_macros('{}>{}'.format(plg.title, para))
    plg.app.add_task(plg)
    start = time()
    transint = '2int' in plg.note and ips.dtype in (np.uint8, np.uint16)
    transfloat = '2float' in plg.note and not ips.dtype in (np.complex128, np.float32, np.float64)
    if transint: 
        buf = img.astype(np.int32)
        src = src.astype(np.int32)
    if transfloat: 
        buf = img.astype(np.float32)
        src = src.astype(np.float32)
    rst = process_channels(plg, ips, src, buf if transint or transfloat else img, para)
    if not img is rst and not rst is None:
        imgrange = {np.uint8:(0,255), np.uint16:(0, 65535)}[img.dtype.type]
        np.clip(rst, imgrange[0], imgrange[1], out=img)
    if 'auto_msk' in plg.note and not ips.mask('out') is None:
        msk = ips.mask('out')
        img[msk] = src[msk]
    plg.app.info('%s: cost %.3fs'%(plg.title, time()-start))
    ips.update()
    plg.app.remove_task(plg)
    if not callafter in (None, 'no record'):callafter()
    
def process_stack(plg, ips, src, imgs, para, callafter=None):
    plg.app.record_macros('{}>{}'.format(plg.title, para))
    plg.app.add_task(plg)
    start = time()
    transint = '2int' in plg.note and ips.dtype in (np.uint8, np.uint16)
    transfloat = '2float' in plg.note and not ips.dtype in (np.complex128, np.float32, np.float64)
    if transint: 
        buf =  imgs[0].astype(np.int32)
        src = src.astype(np.int32)
    elif transfloat: 
        buf = imgs[0].astype(np.float32)
        src = src.astype(np.float32)
    else: src = src * 1

    for i,n in zip(imgs,list(range(len(imgs)))):
        #sleep(0.5)
        plg.progress(n, len(imgs))
        if 'auto_snap' in plg.note : src[:] = i
        if transint or transfloat: buf[:] = i
        rst = process_channels(plg, ips, src, buf if transint or transfloat else i, para)
        if not i is rst and not rst is None:
            imgrange = {np.uint8:(0,255), np.uint16:(0,65535)}[i.dtype.type]
            np.clip(rst, imgrange[0], imgrange[1], out=i)
        if 'auto_msk' in plg.note and not ips.mask() is None:
            msk = ips.mask('out')
            i[msk] = src[msk]
    plg.app.info('%s: cost %.3fs'%(ips.title, time()-start))
    ips.update()
    plg.app.remove_task(plg)
    if not callafter is None:callafter()
    

class Filter:
    title = 'Filter'
    modal = True
    asyn = True
    note = []
    'all, 8-bit, 16-bit, int, rgb, float, not_channel, not_slice, req_roi, auto_snap, auto_msk, preview, 2int, 2float'
    para = None
    view = None
    prgs = None

    def __init__(self, ips=None): pass
        
    def progress(self, i, n): self.prgs = int(i*100/n)

    def show(self):
        preview = lambda para, ips=self.ips: self.preview(ips, para) or ips.update()
        return self.app.show_para(self.title, self.para, self.view, preview, 
            on_ok=lambda : self.ok(self.ips), on_help=self.on_help,
            on_cancel=lambda : self.cancel(self.ips) or self.ips.update(), 
            preview='preview' in self.note, modal=self.modal)
    
    def run(self, ips, snap, img, para = None):
        return 255-img
        
    def check(self, ips):
        note = self.note
        if ips == None:
            return self.app.alert('No image opened!')
            return False
        elif 'req_roi' in note and ips.roi == None:
            return self.app.alert('No Roi found!')
        elif not 'all' in note:
            if ips.dtype==np.uint8 and ips.channels==3 and not 'rgb' in note:
                return self.app.alert('Do not surport rgb image')
            elif ips.dtype==np.uint8 and ips.channels==1 and not '8-bit' in note:
                return self.app.alert('Do not surport 8-bit image')
            elif ips.dtype==np.uint16 and not '16-bit' in note:
                return self.app.alert('Do not surport 16-bit uint image')
            elif ips.dtype in [np.int32, np.int64] and not 'int' in note:
                return self.app.alert('Do not surport int image')
            elif ips.dtype in [np.float32, np.float64] and not 'float' in note:
                return self.app.alert('Do not surport float image')
        return True
        
    def preview(self, ips, para):
        process_one(self, ips, ips.snap, ips.img, para, 'no record')
        
    def load(self, ips):return True
          
    def ok(self, ips, para=None, callafter=None):
        if para == None:
            para = self.para
            if not 'not_slice' in self.note and ips.slices>1:
                if para == None:para = {}
            if para!=None and 'stack' in para:del para['stack']
        # = WidgetsManager.getref('Macros Recorder')
        if ips.slices==1 or 'not_slice' in self.note:
            # process_one(self, ips, ips.snap, ips.img, para)
            if self.asyn and self.app.asyn:
                threading.Thread(target = process_one, args = 
                    (self, ips, ips.snap, ips.img, para, callafter)).start()
            else: process_one(self, ips, ips.snap, ips.img, para, callafter)
            # if win!=None: win.write('{}>{}'.format(self.title, para))
            
        elif ips.slices>1:
            has, rst = 'stack' in para, None
            if not has:
                rst = self.app.yes_no('Run every slice in current stacks?')
            if 'auto_snap' in self.note and self.modal:ips.reset()
            if has and para['stack'] or rst == 'yes':
                para['stack'] = True
                #process_stack(self, ips, ips.snap, ips.imgs, para)
                if self.asyn and self.app.asyn:
                    threading.Thread(target = process_stack, args = 
                        (self, ips, ips.snap, ips.imgs, para, callafter)).start()
                else: process_stack(self, ips, ips.snap, ips.imgs, para, callafter)
                
            elif has and not para['stack'] or rst == 'no': 
                para['stack'] = False
                #process_one(self, ips, ips.snap, ips.img, para)
                if self.asyn and self.app.asyn:
                    threading.Thread(target = process_one, args = 
                        (self, ips, ips.snap, ips.img, para, callafter)).start()
                else: process_one(self, ips, ips.snap, ips.img, para, callafter)
            elif rst == 'cancel': pass
        #ips.update()
        
    def on_help(self):
        self.app.show_md(self.__doc__ or 'No Document!', self.title)

    def cancel(self, ips):
        if 'auto_snap' in self.note:
            ips.img[:] = ips.snap
            ips.update()
            
    def start(self, app, para=None, callafter=None):
        self.app, self.ips = app, app.get_img()
        if not self.check(self.ips):return
        if not self.load(self.ips):return
        if 'auto_snap' in self.note:self.ips.snapshot()
        
        if para!=None:
            self.ok(self.ips, para, callafter)
        elif self.view==None:
            if not self.__class__.show is Filter.show:
                if self.show():
                    self.ok(self.ips, para, callafter)
            else: self.ok(self.ips, para, callafter)
        elif self.modal:
            if self.show():
                self.ok(self.ips, None, callafter)
            else:self.cancel(self.ips)
        else: self.show()

    def __del__(self):
        print('filter del')



from ..advanced import dataio
from skimage.io import imread, imsave

for i in ('bmp', 'jpg', 'tif'):
	dataio.ReaderManager.add(i, imread, 'img')
	dataio.WriterManager.add(i, imsave, 'img')

class OpenFile(dataio.Reader):
    title = 'Open'

    def load(self):
        self.filt = [i for i in sorted(dataio.ReaderManager.names())]
        return True

class SaveImage(dataio.ImageWriter):
	title = 'Save Image'

	def load(self, ips):
		self.filt = [i for i in sorted(dataio.WriterManager.names())]
		return True


import tkinter as tk
from tkinter import ttk
import numpy as np
from numpy import ndarray
from scitk.canvas.canvas import Canvas
from sciapp.object.image import Image
from sciapp.action import Tool, ImageTool, ShapeTool

class ICanvas(Canvas):
    def __init__(self, parent, autofit=False):
        Canvas.__init__(self, parent, autofit)
        self.images.append(Image())
        #self.images[0].back = Image()
##        self.Bind(wx.EVT_IDLE, self.on_idle)

    def get_obj_tol(self):
        return self.image, ImageTool.default
        
    def set_img(self, img, b=False):
        isarr = isinstance(img, ndarray)
        if b and not isarr: self.images[0].back = img
        if not b and not isarr: self.images[0] = img
        if b and isarr: self.images[0].back = Image([img])
        if not b and isarr: self.images[0].img = img
        if not b: self.images[0].reset()
        if b and self.images[0].back: 
            self.images[0].back.reset()
        self.update_box()
        self.update()

    def set_log(self, log, b=False):
        if b: self.back.log = log
        else: self.image.log = log
        
    def set_rg(self, rg, b=False):
        if b: self.back.rg = rg
        else: self.image.rg = rg
        
    def set_lut(self, lut, b=False):
        if b: self.back.lut = lut
        else: self.image.lut = lut

    def set_cn(self, cn, b=False):
        if b: self.back.cn = cn
        else: self.image.cn = cn

    def set_mode(self, mode):
        self.image.mode = mode

    def set_tool(self, tool):
        self.tool = tool

    @property
    def image(self): return self.images[0]

    @property
    def back(self): return self.images[0].back

    def draw_ruler(self, dc, f, **key):
        dc.SetPen(wx.Pen((255,255,255), width=2, style=wx.SOLID))
        conbox, winbox, oribox = key['conbox'], key['winbox'], key['oribox']
        x1 = max(conbox[0], winbox[0])+5
        x2 = min(conbox[2], winbox[2])-5
        pixs = (x2-x1+10)*(oribox[2]-oribox[0])/10.0/(conbox[2]-conbox[0])
        h = min(conbox[3], winbox[3])-5
        dc.DrawLineList([(x1,h,x2,h)])
        dc.DrawLineList([(i,h,i,h-5) for i in np.linspace(x1, x2, 11)])
        dc.SetTextForeground((255,255,255))
        k, unit = self.image.unit
        text = 'Unit = %.1f %s'%(k*pixs, unit)
        dw, dh = dc.GetTextExtent(text)
        dc.DrawText(text, (x2-dw, h-10-dh))

    def on_idle(self, event):
        if self.image.unit == (1, 'pix'):
            if 'unit' in self.marks: del self.marks['unit']
        else: self.marks['unit'] = self.draw_ruler
        if self.image.roi is None:
            if 'roi' in self.marks: del self.marks['roi']
        else: self.marks['roi'] = self.image.roi
        if self.image.mark is None:
            if 'mark' in self.marks: del self.marks['mark']
        elif self.image.mark.dtype=='layers':
            if self.image.cur in self.image.mark.body:
                self.marks['mark'] = self.image.mark.body[self.image.cur]
            elif 'mark' in self.marks: del self.marks['mark']
        else: self.marks['mark'] = self.image.mark
        Canvas.on_idle(self, event)

class VCanvas(Canvas):
    def __init__(self, parent, autofit=False, ingrade=True, up=True):
        Canvas.__init__(self, parent, autofit, ingrade, up)

    def get_obj_tol(self):
        return self.shape, ShapeTool.default

    def set_shp(self, shp):
        self.marks['shape'] = shp
        self.update()

    def set_tool(self, tool): self.tool = tool

    @property
    def shape(self): 
        if not 'shape' in self.marks: return None
        return self.marks['shape']

class SCanvas(ttk.Frame):
    def __init__(self, parent, autofit=False):
        ttk.Frame .__init__(self, parent)
        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
        self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )

        sizer = wx.BoxSizer( wx.VERTICAL )
        self.lab_info = wx.StaticText( self, wx.ID_ANY,
            'information', wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lab_info.Wrap( -1 )
        # self.lab_info.Hide()
        sizer.Add( self.lab_info, 0, wx.ALL, 0 )
        
        self.canvas = VCanvas(self, autofit = autofit)
        sizer.Add( self.canvas, 1, wx.EXPAND |wx.ALL, 0 )
        self.SetSizer(sizer)
        self.set_shp = self.canvas.set_shp

        self.Bind(wx.EVT_IDLE, self.on_idle)

    def on_idle(self, event):
        if self.shape is None: return
        if self.shape.dirty: self.update()
        self.shape.dirty = False

    def update(self):
        if self.shape is None: return
        self.canvas.update()
        if self.lab_info.GetLabel()!=self.shape.info:
            self.lab_info.SetLabel(self.shape.info)

    def Fit(self):
        wx.Panel.Fit(self)
        self.GetParent().Fit()

    @property
    def shape(self): return self.canvas.shape

    @property
    def name(self): return self.canvas.shape.name


class MCanvas(ttk.Frame):
    def __init__(self, parent=None, autofit=False):
        ttk.Frame.__init__ ( self, parent)
        
#        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
##       [ToDo:]设置背景颜色需要为白色
        
        sizer = ttk.Frame(self)
        sizer.pack(expand = True, fill=tk.BOTH)
        self.lab_info = ttk.Label( sizer, text = 'information')
        self.lab_info.pack()
        #self.lab_info.Hide()
        
        self.canvas = ICanvas(sizer, autofit = autofit)
        self.canvas.pack(expand = True, fill=tk.BOTH)

##        self.sli_chan = wx.Slider( self, wx.ID_ANY, 0, 0, 100, wx.DefaultPosition, wx.DefaultSize, 
##            wx.SL_HORIZONTAL| wx.SL_SELRANGE| wx.SL_TOP)
##        sizer.Add( self.sli_chan, 0, wx.ALL|wx.EXPAND, 0 )
##        self.sli_chan.SetMaxSize( wx.Size( -1,18 ) )
##        self.sli_chan.Hide()
##
##        self.sli_page = wx.ScrollBar( self, wx.ID_ANY,
##                                  wx.DefaultPosition, wx.DefaultSize, wx.SB_HORIZONTAL)
##        self.sli_page.SetScrollbar(0,0,0,0, refresh=True)
##        sizer.Add( self.sli_page, 0, wx.ALL|wx.EXPAND, 0 )
##        self.sli_page.Hide()

##        self.sli_page.Bind(wx.EVT_SCROLL, self.on_scroll)
##        self.sli_chan.Bind(wx.EVT_SCROLL, self.on_scroll)
##        self.Bind(wx.EVT_IDLE, self.on_idle)
        #self.Fit()
        self.set_rg = self.canvas.set_rg
        self.set_lut = self.canvas.set_rg
        self.set_log = self.canvas.set_log
        self.set_mode = self.canvas.set_mode
        self.set_tool = self.canvas.set_tool

        self.chans, self.pages, self.cn, self.cur = -1, -1, -1, -1

    def set_img(self, img, b=False):
        self.canvas.set_img(img, b)
        self.canvas.update_box()
        self.update()

    def set_cn(self, cn, b=False):
        self.canvas.set_cn(cn, b)
        self.update()
        
    @property
    def image(self): return self.canvas.image

    @property
    def back(self): return self.canvas.back

    @property
    def name(self): return self.canvas.image.name

    def set_imgs(self, imgs, b=False):
        if b: self.canvas.back.set_imgs(imgs)
        else: self.canvas.image.set_imgs(imgs)
        self.canvas.update_box()
        self.update()

    def Fit(self):
        wx.Panel.Fit(self)
        self.GetParent().Fit()
        
    def slider(self):
        
        return 
        
        slices = self.image.slices
        channels = self.image.channels
        if slices != self.pages or self.image.cur != self.cur:
            print('set slices')
            if slices==1 and self.sli_page.Shown:
                self.sli_page.Hide()
            if slices>1 and not self.sli_page.Shown:
                self.sli_page.Show()
            self.sli_page.SetScrollbar(self.image.cur, 0, slices-1, 0, True)
            self.pages, self.cur = slices, self.image.cur
        if channels != self.chans or self.cn != self.image.cn:
            print('set channels')
            if not isinstance(self.image.cn, int) and self.sli_chan.Shown:
                self.sli_chan.Hide()
            if isinstance(self.image.cn, int) and channels>1:
                if not self.sli_chan.Shown: self.sli_chan.Show()
                self.sli_chan.SetValue(self.image.cn)
            self.sli_chan.SetMax(channels-1)
            self.chans, self.cn = channels, self.image.cn

    def update(self):
        
        return 
        
        if self.image.img is None: return
        self.slider()
        if self.lab_info.GetLabel()!=self.image.info:
            self.lab_info.SetLabel(self.image.info)
        # self.canvas.update()
        self.Layout()

    def on_scroll(self, event):
        self.image.cur = self.sli_page.GetThumbPosition()
        self.image.dirty = True
        if isinstance(self.image.cn, int):
            self.image.cn = self.sli_chan.GetValue()
        self.canvas.on_idle(event)
    
    def on_idle(self, event):
        image, info = self.image, self.lab_info.GetLabel()
        imgs = image.slices, image.channels, image.cn, image.cur
        selfs = self.pages ,self.chans, self.cn, self.cur
        if imgs != selfs or info!=self.image.info: self.update()

    def __del__(self):
        print('canvas panel del')

if __name__=='__main__':
    from skimage.data import camera, astronaut
    from skimage.io import imread
    
    app = wx.App()
    frame = wx.Frame(None, title='MCanvas')
    mc = MCanvas(frame, autofit=False)
    mc.set_img(astronaut())
    mc.set_cn(0)
    frame.Show()
    app.MainLoop()




import time
import tkinter
import numpy as np
from OpenGL import GL
from OpenGL import GLU
from pyopengltk import OpenGLFrame

def circle(x, y, r, n):
    r=abs(r)
    print(r)
    
    theta = np.linspace(0, 2*np.pi, n)
    x = x + r * np.cos(theta)
    y = y + r * np.sin(theta)
    return x, y
class Canvas3D(OpenGLFrame):
    def __init__(self,master,width=200,height=200):
        super(). __init__(master,width=width,height=height)
        self.run=True
    def initgl(self):
        
        """Initalize gl states when the frame is created"""
        GL.glViewport(0, 0, self.width, self.height)
        
        #GL.glClearColor(0.0, 1.0, 0.0, 0.0)    
        self.start = time.time()
        self.nframes = 0
        GLU.gluOrtho2D(-5.0, 5.0, -5.0, 5.0)    # 设置显示范围

    def redraw(self):
        """Render a single frame"""
        if self.run==True:
            self.draw()
    def draw(self):
        GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
        
        GL.glBegin(GL.GL_POINTS)
        
        x, y = circle(0, 0, 2+1.5*np.random.randn(),100)
        GL.glColor3f(1.0, 0.0, 0.0)
        for x_, y_ in zip(x, y):
            GL.glVertex2f(x_, y_)
        GL.glEnd()                             # 此次结束
        GL.glFlush() 
        
        tm = time.time() - self.start
        self.nframes += 1
        print("fps",self.nframes / tm, end="\n" )
        
    def setData(self):
        pass
    def stopOrGo(self):
        self.run=not self.run

def test():
    global b
    b.config(text='eeeeee')
if __name__ == '__main__':
    root = tkinter.Tk()
    frame=tkinter.Frame(root)
    frame.pack()
    app = Canvas3D(frame, width=200, height=200)
    b=tkinter.Button(text='hahahahaha',command=app.stopOrGo)
    b.pack()
    
    app.pack(fill=tkinter.BOTH, expand=tkinter.YES)
    app.animate = 1000 // 100# 这样可以控制刷新速度。
    app.after(1000, app.printContext)
    
    app.mainloop()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
""""""

import tkinter as tk
from tkinter import ttk

# http://docs.wxwidgets.org/3.1/classwx_choicebook.html

__title__ = "ChoiceBook"
__version__ = "1.0.2"
__author__ = "DeflatedPickle"


class BaseChoiceBook(ttk.Frame):
    """
            -----DESCRIPTION-----
    The ChoiceBook allows for navigation through added frames by using a Combobox.
            -----USAGE-----
    choiceBook = ChoiceBook(parent)
    choiceBook.pack()
    frame = ttk.Frame(choiceBook.frame)
    choiceBook.add(child=frame, label="Frame")
            -----PARAMETERS-----
    combobox_position  = The position of the Combobox.
            -----CONTENTS-----
    ---VARIABLES---
    parent             = The parent of the widget.
    _combobox_position = The position of the Combobox.
    _page_list         = A list of the pages.
    _label_list        = A list of the labels.
    ---TKINTER VARIABLES---
    _variable          = The current page of the widget.
    ---WIDGETS---
    self
    _combobox          = The Combobox used to change page.
    frame              = Holds the currently shown Frame.
    ---FUNCTIONS---
    _change_page()     = Changes the page to the selected one.
    add()              = Adds a new page to the widget.
    """
    def __init__(self, parent, combobox_position="top", *args):
        ttk.Frame.__init__(self, parent, *args)
        self.parent = parent
        self._combobox_position = combobox_position

        self._page_list = []
        self._label_list = []

        self._variable = tk.StringVar()

        self._combobox = ttk.Combobox(self, state="readonly", textvariable=self._variable)
        self._combobox.pack(side=combobox_position, fill="x")
        self._combobox.bind("<<ComboboxSelected>>", self._change_page, "+")

        self.frame = ttk.Frame(self)
        self.frame.pack(fill="both", expand=True)

    def _change_page(self, event=None):
        for i in range(len(self._page_list)):
            self._page_list[i].pack_forget()

        self._page_list[self._label_list.index(self._variable.get())].pack(fill="both", expand=True)

    def add(self, child=None, label=""):
        """Adds a new page to the ChoiceBook."""
        self._page_list.append(child)
        self._label_list.append(label)
        self._variable.set(self._label_list[0])
        self._combobox.configure(values=self._label_list)

        self._change_page()

##################################################

if __name__ == "__main__":
    root = tk.Tk()
    cbook = BaseChoiceBook(root)
    cbook.pack(expand=True, padx=5, pady=5)

    frame1 = ttk.Frame(cbook.frame)
    for i in range(3):
        ttk.Button(frame1, text=i).pack(side="left")
    frame2 = ttk.Frame(cbook.frame)
    ttk.Checkbutton(frame2, text="Checkbutton").pack()
    frame3 = ttk.Frame(cbook.frame)
    ttk.Label(frame3, text="Frame 3").pack(side="bottom")

    cbook.add(child=frame1, label="Frame1")
    cbook.add(child=frame2, label="Frame2")
    cbook.add(child=frame3, label="Frame3")
    root.mainloop()


class Macros:
    def __init__(self, title, cmds):
        self.title = title
        self.cmds = cmds
        
    def __call__(self): return self
        
    def start(self, app, para=None, callafter=None):
        app.run_macros(self.cmds, callafter)


import numpy as np

def cross(winbox, conbox):
    two = np.array([winbox, conbox])
    x1, y1 = two[:,:2].max(axis=0)
    x2, y2 = two[:,2:].min(axis=0)
    return [x1, y1, x2, y2]

def merge(winbox, conbox):
    two = np.array([winbox, conbox])
    x1, y1 = two[:,:2].min(axis=0)
    x2, y2 = two[:,2:].max(axis=0)
    return [x1, y1, x2, y2]

def multiply(rect, kx, ky):
    return rect * [kx, ky, kx, ky]

def layx(winbox, conbox):
    conw = conbox[2]-conbox[0]
    winw = winbox[2]-winbox[0]   
    if conw<winw:
        mid = (winbox[0]+winbox[2])/2
        conbox[0] = mid-conw/2
        conbox[2] = mid+conw/2
    elif conbox[0] > winbox[0]:
        conbox[0] = winbox[0]
        conbox[2] = conbox[0] + conw
    elif conbox[2] < winbox[2]:
        conbox[2] = winbox[2]
        conbox[0] = conbox[2] - conw

def layy(winbox, conbox):
    winh = winbox[3]-winbox[1]
    conh = conbox[3]-conbox[1] 
    if conh<winh:
        mid = (winbox[1]+winbox[3])/2
        conbox[1] = mid-conh/2
        conbox[3] = mid+conh/2
    elif conbox[1] > winbox[1]:
        conbox[1] = winbox[1]
        conbox[3] = conbox[1] + conh
    elif conbox[3] < winbox[3]:
        conbox[3] = winbox[3]
        conbox[1] = conbox[3] - conh

def lay(winbox, conbox):
    layx(winbox, conbox)
    layy(winbox, conbox)

def like(ori, cont, cell):
    kx = (cont[2]-cont[0])/(ori[2]-ori[0])
    ky = (cont[3]-cont[1])/(ori[3]-ori[1])
    ox = cont[0] - ori[0]*kx
    oy = cont[1] - ori[1]*kx
    return [cell[0]*kx+ox, cell[1]*ky+oy, 
        cell[2]*kx+ox, cell[3]*kx+oy]

def mat(ori, con, cell, cros):
    kx = (ori[2]-ori[0])/(con[2]-con[0])
    ky = (ori[3]-ori[1])/(con[3]-con[1])
    ox = (cros[1]-cell[1])*ky
    oy = (cros[0]-cell[0])*kx
    return (ox, oy), (kx, ky)


import json, os.path as osp

class Manager:
    def __init__(self, unique=True, path=None):
        self.objs, self.unique, self.path = [], unique, path
        if not path is None: self.read(path)

    def add(self, name, obj, tag=None):
        if self.unique: self.remove(name, tag)
        self.objs.insert(0, (name, obj, tag))

    def active(self, name=None, tag=None, obj=None):
        objs = self.gets(name, tag, obj)
        for i in objs: self.objs.remove(i)
        for i in objs: self.objs.insert(0, i)

    def set(self, name, obj, tag=None):
        self.remove(name, tag)
        self.objs.insert(0, (name, obj, tag))

    def adds(self, objs): 
        for i in objs: self.add(*i)

    def get(self, name=None, tag=None, obj=None):
        rst = self.gets(name, tag, obj)
        return None if len(rst)==0 else rst[0][1]

    def gets(self, name=None, tag=None, obj=None):
        rst = [i for i in self.objs if name is None or name == i[0]]
        rst = [i for i in rst if obj is None or obj is i[1]]
        return [i for i in rst if tag is None or tag == i[2]]
        
    def has(self, name=None, tag=None, obj=None):
        return len(self.gets(name, tag, obj))>0
        
    def remove(self, name=None, tag=None, obj=None):
        for i in self.gets(name, tag, obj): self.objs.remove(i)

    def names(self, tag=None):
        return [i[0] for i in self.gets(tag=tag)]
        
    def name(self, name):
        names = self.names()
        if not name in names : return name
        for i in range(1, 100) : 
            n = "%s-%s"%(name, i)
            if not n in names: return n

    def write(self, path=None):
        with open(path or self.path, 'w') as f:
            f.write(json.dumps(self.objs))

    def read(self, path):
        self.path = path
        if not osp.exists(path): return
        with open(path) as f:
            self.adds(json.loads(f.read()))
        return self


from ..advanced import Filter
from scipy.ndimage import gaussian_filter

class Gaussian(Filter):
    title = 'Gaussian'
    note = ['all', 'auto_msk', 'auto_snap','preview']
    para = {'sigma':2}
    view = [(float, 'sigma', (0,30), 1,  'sigma', 'pix')]
    
    def run(self, ips, snap, img, para = None):
        gaussian_filter(snap, para['sigma'], output=img)